<?php
/* DAT/SITEMAP.PHP - This creates and updates the file sitemap.xml.

   It's the initial version and all the strings and HTML are inline; so far 
   that is alright (easy to manage and all that). The interface is extremely 
   simple (which is in our name, afterall). It only lacks support for inputs 
   for <changefreq> and <priority> (which have to be changed manually).

   It turns out to be rather easy to manipulate the XML file after we created 
   the parse_xml_file() function as an experiment. There is no need for a 
   complicated Class to edit XML if you just convert the XML to an array and 
   back (although here, the function array_to_xml() is specific to the sitemap 
   format).
*/


function sitemap() {

	$html = html();
	$done = $html['done'];
	$done = str_replace('&op=sitemap','',$done);
	$url = $html['host'].'?op=sitemap';
	$xmlns = 'xmlns="http://www.sitemaps.org/schemas/sitemap/0.9"';
	$status = '';


	if (isset($_GET['view'])) {			// view file

		$sitemap = file_get_contents('sitemap.xml');
		echo "<pre class='sitemap'>".htmlentities($sitemap)."</pre>";
		echo "<form method='post' action='$url'>";
		echo "<button name='button' value='go back'>back</button>";
		echo "</form>";
		return;
	}

	if (isset($_GET['create'])) {			// create file

		$sitemap = sitemap_create($xmlns);
		$status = 'The sitemap.xml file has been created.';
		if (!$sitemap) 
			$status = error('php_error');
	}

	if (isset($_GET['update'])) {			// update file

// the update form has section names as inputs (checkboxes)
		foreach ($_POST as $section => $unused) {
			if ($section == 'button')	// skip the button
				continue;
			$sections[] = $section;
		}

		if (!isset($sections)) {
			$status = 'No sections were selected.';
		}
		else {
			if (filesize('sitemap.xml') == 0)
				sitemap_create($xmlns);
			$sitemap = sitemap_update($sections,$xmlns);	
			$status = 'The sitemap.xml file has been updated.';
			if (!$sitemap)
				$status = '';
		}
	}

	if (isset($_GET['unlink'])) {
		file_put_contents('sitemap.xml','');
	}

	/* introduction text and forms */

echo <<<HTML
<div class='menu'>
<span>
<span class='help' style='cursor:default;'></span>
<p>The <b>sitemap</b> command will create or update the <tt>sitemap.xml</tt> 
file in the base directory.<br><br>To update the file check which section has 
had recent posts and then click <a>update</a>.<br><br>
If a section has been added or removed click <a>reset</a> to create a new 
<tt>sitemap.xml</tt> file.<br><br>Click <a>view</a> to view the current 
<tt>sitemap.xml</tt> file.
</p>
</span>
</div>
HTML;

	if (!is_file('sitemap.xml')) {
echo <<<HTML
<p>The file <tt>sitemap.xml</tt> does not exist. It needs to exist as an empty 
file and needs PHP write permissions.<br><br>
<tt>touch sitemap.xml; chmod 646 sitemap.xml</tt></p>
<form method='post' action='$done'>
<input type='submit' name='button' value='done'>
</form>
HTML;
		return;
	}

	if (!is_writeable('sitemap.xml')) {
echo <<<HTML
<p>The file <tt>sitemap.xml</tt> exists but is not writeable.<br><br>
<tt>chmod 646 sitemap.xml</tt></p>
<form method='post' action='$done'>
<input type='submit' name='button' value='done'>
</form>
HTML;
		return;
	}

echo <<<HTML
<p><b>Update the <tt>sitemap.xml</tt> File</b> &nbsp;<span class='status'>$status</span></p>
<p>Select the sections that have had recent posts for update.</p>
<form method='post' action='$url&amp;update'>
HTML;

	$sections = config('sections');
	foreach ($sections as $name => $section) {
		if (isset($section['disabled']))
			continue;
		if ($name == '')
			$name = 'root';
echo <<<HTML
<label style='width:88px;display:inline-block;' for='$name'>$name</label>
<input type='checkbox' name='$name'><br>
HTML;
	}

echo <<<HTML
<p><button name='button' value='sitemap' title='update sitemap'>update</button>
Update sitemap for the selected sections.</p>
</form>
<form method='post' action='$url&amp;create'>
<p><button name='button' value='sitemap' title='reset sitemap'>reset</button> 
Reset sitemap for all sections. (If <tt>sections.ini</tt> changed.)</p>
</form>
<form method='post' action='$url&amp;view'>
<p><button name='button' value='sitemap' title='view sitemap'>view</button>
View sitemap.</p>
</form>
<form method='post' action='$done'>
<p><button name='button' value='done' title='done with sitemap'>done</button>
Done with sitemap.</p>
</form>
HTML;

	if (filesize('sitemap.xml') == 0) {
# waffle...
/*		sitemap_create($xmlns);
echo <<<HTML
<p>A new sitemap has been created with the current date as it's 
<tt>lastmod</tt>...
HTML;
*/
echo <<<HTML
<p>A sitemap exists and is empty. Use <tt>reset</tt> to create a sitemap. 
After adding posts use <tt>update</tt> to update the <tt>lastmod</tt> for 
the section(s) that have been updated. (This is the first version and there 
is no API yet. The next step will be to create an API to allow automatic 
updates.)</p>
HTML;
	}
}


// Admin API starts here

/* sitemap_create - create our sitemap based on the configured sections */

// expects the file 'sitemap.xml' to exist and be writeable

function sitemap_create($attr) {

	$siteurl = config('siteurl');
	$lastmod = date('Y-m-d');

$sitexml = <<<XML
<?xml version="1.0" encoding="UTF-8"?>
<urlset $attr>
  <url>
     <loc>$siteurl</loc>
     <lastmod>$lastmod</lastmod>
     <changefreq>weekly</changefreq>
     <priority>0.8</priority>
  </url>

XML;
	$sections = config('sections');
	foreach ($sections as $name => $section) {
		if ($name == '')
			continue;
		if (isset($section['disabled']))
			continue;
$sitexml .= <<<XML
  <url>
     <loc>$siteurl?$name</loc>
     <lastmod>$lastmod</lastmod>
     <changefreq>weekly</changefreq>
     <priority>0.5</priority>
  </url>

XML;
	}

	$sitexml .= '</urlset>';
	return file_put_contents('sitemap.xml',$sitexml);
}

/* sitemap_update - update our sitemap based on the changed sections */

function sitemap_update($sections, $xmlns) {

	$data = parse_xml_file('sitemap.xml',$name);
	if (!$data)
		return FALSE;

	foreach ($data['url'] as $i => $val) {
		foreach ($sections as $section) {
			if ($section == 'root') {
				$data['url'][0]['lastmod'] = date('Y-m-d');
				continue;
			}
			if (strstr($val['loc'],$section))
				$data['url'][$i]['lastmod'] = date('Y-m-d');
		}
	}

	$sitexml = array_to_xml($data,'urlset',$xmlns);
	return file_put_contents('sitemap.xml',$sitexml);
}


// XML API starts here

/* array_to_xml - convert assoc. array to XML format */

// kind of specific to the sitemap format for now

function array_to_xml($data, $name, $attr = '') {

	$xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
	$xml .= "<$name $attr>\n";
	foreach ($data as $item => $dat) {
		foreach ($dat as $d) {
			$xml .= "  <$item>\n";
			foreach ($d as $k => $v)
				$xml .= "    <$k>$v</$k>\n";
			$xml .= "  </$item>\n";
		}
	}
	$xml .= "</$name>";

	return $xml;
}

/* parse_xml_file - read XML file into an associative array */

// $name will be set to the XML open tag

function parse_xml_file($file, &$name) {

	$data = file_get_contents($file);

	$xml_parser = xml_parser_create();
	xml_parse_into_struct($xml_parser,$data,$values,$index);
	$xml_error = xml_get_error_code($xml_parser);
	if ($xml_error) {
		echo '<p><b>parse_xml_file</b> ';
		echo xml_error_string($xml_error).'</p>';
		return array();
	}
	xml_parser_free($xml_parser);

	$open = array_shift($values);
	$name = strtolower($open['tag']);

	$a = xml_get_section($values,$index);

	return $a;
}

function xml_get_section(&$values,$index) {

	$a = array();

	while ($val = array_shift($values)) {
		if ($val['type'] == 'complete') {
			$v = strtolower($val['tag']);
			$d = (isset($val['value'])) ? $val['value'] : '';
			$a[$v] = trim($d);
		}
		if ($val['type'] == 'close') {
			break;
		}
		if ($val['type'] == 'open') {
			$v = strtolower($val['tag']);
			$a[$v][] = xml_get_section($values,$index);
		}
	}

	return $a;
}
