<?php
/*
	"NCSA httpd 1.3 would return some boring old error/problem message 
	which would often be meaningless to the user, and would provide no 
	means of logging the symptoms which caused it."
	  -- Apache, 'Custom Error Responses'
*/
/*
	"Once you are ready for deployment, you should either disable error 
	reporting completely by setting error_reporting() to 0, or turn off 
	the error display using the php.ini option display_errors, to insulate 
	your code from probing. If you choose to do the latter, you should also 
	define the path to your log file using the error_log ini directive, and 
	turn log_errors on."
	  -- PHP.NET
*/

/* ERROR.PHP - error message handling

This gets included by INDEX.PHP and ADMIN.PHP (the two sole entry points) 
as the very first include; it provides two main things:

	Error and informational messages, both fatal and notices.
	A few server "error" functions.
*/

error_reporting(0);			// if "whitescreen" use Admin

_error();				// initialize on first call
checkbaduser();				// checks banned IPs and Referers
					//  might exit

/* module support functions - see file MODULES */

function modules_set($file) {
	$GLOBALS['modules'][] = $file;
	$file = basename($file);
	$init = '_config_'.str_replace('.php','',$file);
	if (function_exists($init))
		$GLOBALS['functions'][] = $init;
}

function modules_init() {
	foreach ($GLOBALS['functions'] as $name) {
		debug($name);
		$name();
	}
	unset($GLOBALS['functions']);
}

function modules_loaded() {
	foreach ($GLOBALS['modules'] as $name)
		echo "\n<!-- $name -->";
	unset($GLOBALS['modules']);
}
//register_shutdown_function('modules_loaded');
// that will interfere with redirects and file downloads if registered


/* redirect - send browser to a new location */

function redirect($url, $moved = FALSE) {

	debug("'$url'");

	$url = str_replace('&amp;','&',$url);

	$host = '';
	if (strpos($url,'http://') === FALSE) {
		$host = gethost();
		$url = $host.$url;
	}

	if (headers_sent($file,$line)) {
		debug("headers already sent by $file, line $line");
		debug(implode('<br>',headers_list()));
		print("Location: <a href='$url'>$url</a>");
		exit;
	}

	if (config('noredirect'))
		print("Location: <a href='$url'>$url</a>");
	else {
		debuglog(0);
		if ($moved)
			header('HTTP/1.1 301 Moved Permanently'); 
		header("Location: $url");
	}
	exit;
}


/* gethost - return host name */

function gethost() {

	$host = 'http://'.$_SERVER['HTTP_HOST'];
	$host .= dirname($_SERVER['SCRIPT_NAME']);
	return $host.'/';
}


/* checkbaduser - look for match of banned ip or user agent */

// the use of function names in the INI data may seem odd, but it provides 
// yet more configuration possibilities

function checkbaduser() {

	$e = _error();

	if (!function_exists($f=_error('user_agent')))
		$f = NULL;
	if (isset($_SERVER['HTTP_USER_AGENT']))
		foreach ($e['HTTP_USER_AGENT'] as $_)
			if (strstr($_SERVER['HTTP_USER_AGENT'],$_)) {
				if (!$f) return TRUE;
				$f();
			}

	if (!function_exists($f=_error('remote_addr')))
		$f = NULL;
	if (isset($_SERVER['REMOTE_ADDR']))
		foreach ($e['REMOTE_ADDR'] as $_)
			if (strstr($_SERVER['REMOTE_ADDR'],$_)) {
				if (!$f) return TRUE;
				$f();
			}

	// some referers also suck

	if (!function_exists($f=_error('referer')))
		$f = NULL;
	if (isset($_SERVER['HTTP_REFERER']))
		foreach ($e['HTTP_REFERER'] as $_)
			if (strstr($_SERVER['HTTP_REFERER'],$_)) {
				if (!$f) return TRUE;
				$f();
			}
}


/* checkbadinput - look for "banned words" */

function checkbadinput($data, $exit = 1) {

	if (!function_exists($f=_error('ban_words')))
		$f = NULL;
	if ($words = _error('BAN_WORDS'))
		foreach ($words as $k => $v) {
			if (preg_match('/^\/.*\/$/',$v))
				$m = preg_match($v,$data);
			else
				$m = stristr($data,$v);
			if ($m) {
				if (!$f) return TRUE;
				$f();
			}
		}

	return FALSE;
}

/* server error page functions */

function notfound($place = NULL) {
	if (!$place) $place = gethost();

	header('HTTP/1.1 404 Not Found');
	$f = config('file404');
	if ($f)
		readfile($f);
	else
echo "<!DOCTYPE HTML>
<html><head><title>404 Not Found</title></head><body>
<h1>Not Found</h1>
<p>The requested URL was not found on this server.</p>
<hr>
<ADDRESS>
Web Server at $place
</ADDRESS>
</body>
</html>
";
	exit;
}

function forbidden() {
	header("HTTP/1.1 403 Forbidden");
	$f = config('file403');
	if ($f)
		readfile($f);
	else
echo '<!DOCTYPE HTML>
<HTML><HEAD><TITLE>403 Forbidden</TITLE>
</HEAD><BODY>
<H1>That is Forbidden</H1>
You have done something wrong.
</BODY>
</HTML>
';
	exit;
}

function badrequest() {
	header("HTTP/1.1 400 Bad Request");
	$f = config('file400');
	if ($f)
		readfile($f);
	else
echo '<!DOCTYPE HTML>
<HTML><HEAD><TITLE>400 Bad Request</TITLE>
</HEAD><BODY>
<H1>400 Bad Request</H1>
The request you made was invalid.
</BODY>
</HTML>
';
	exit;
}


/* ERROR internal functions */

/* this module's config */

function _error($var = NULL) {
static $error = array();

	if ($error == array()) {
		// this is the INI file format - eventually it will be PHP
		$error = parse_ini_file('error.ini',TRUE);
		return;
	}

	if ($var === NULL)
		return $error;

	if (isset($error[$var]))
		return $error[$var];

	return '';
}
