<?php

/* FILEMODE.PHP - a file based "database" 

   These functions are the same as the MySQL functions except they read/write 
   files. The file formats are the same as the database formats, and the API 
   is identical.

   These functions just work. A map need to be drawn, but for now the 
   CONFIG.INI "dir" settings point to writeable directories (repository) 
   which need no credential or session data.
*/

function dbconnect() {
return TRUE;
}

function dbselect() {
return TRUE;
}

function dbquery() {
return TRUE;
}

function dberror() {
	$dbdir = config('importdir');

	if (!is_dir($dbdir))
		return "directory '$dbdir' does not exist";
	if (!is_writeable($dbdir))
		return "directory '$dbdir' not writeable";
	return "unknown error with directory '$dbdir'";
}

function dblistrecords($id = NULL) {
	$dbdir = config('importdir');
	if ($id)
		return is_file($dbdir.$id.'.txt');
	if (!$dbdir)
		return FALSE;
	$ids = glob($dbdir.'*.txt');
	$ids = str_replace($dbdir,'',$ids);
	$ids = str_replace('.txt','',$ids);
	natsort($ids);
	sort($ids);				// this resets the indexes
	return $ids;
}

function dbreadrecord($id) {
	$dbdir = config('importdir');
	$rec = file_get_contents($dbdir.$id.'.txt');
	if (!$rec) return $rec;
	return stringtorecord($rec);
}

function dbgetrecord($id) {
	$dbdir = config('importdir');
	return file_get_contents($dbdir.$id.'.txt');
}

function dbnewrecord(/*mixed*/ $data) {
	$dbdir = config('importdir');
	if (is_array($data))
		$data = recordtostring($data);
	$ids = dblistrecords();

# gets first one as 2 if there are none
	$i = 1;
	foreach ($ids as $id) {
		if (!is_numeric($id))
			continue;
		if ($id > $i)
			$i = $id;
	}

	$id = $i+1;
	file_put_contents($dbdir.$id.'.txt',$data);

	if (config('allowcomments'))
		commentdir($id,TRUE);

	return $id;
}

function dbputrecord($id, $data) {
	$dbdir = config('importdir');
	if (is_array($data))
		$data = recordtostring($data);
	file_put_contents($dbdir.$id.'.txt',$data);
	return $id;
}

function dbdelrecord($id) {
	$dbdir = config('importdir');
	return unlink($dbdir.$id.'.txt');
}

function dbcountcomments($entryid) {
	$dbdir = commentdir($entryid);
	if (!$dbdir)
		return 0;
	$ids = glob($dbdir.'*.txt');
	return count($ids);
}

function dblistcomments($entryid) {
	$dbdir = commentdir($entryid);
	if (!$dbdir)
		return array();
	$ids = glob($dbdir.'*.txt');
	natsort($ids);
	sort($ids);				// this resets the indexes
	return $ids;
}

function dbreadcomment($id) {
//this gets the full path to the file
	$data = file_get_contents($id);
	if ($data) {
		$record = stringtorecord($data);
		$record['id'] = str_replace('.txt','',basename($id));
		return $record;
	}
	return $data;
}

function dbgetcomment($id) {
//this gets the full path to the file
	return file_get_contents($id);
}

function dbwritecomment($entryid, $record) {
	$dbdir = commentdir($entryid);
	if (!$dbdir)
		return FALSE;
	$data = recordtostring($record);
	$ids = dblistcomments($entryid);
	$ids = str_replace($dbdir,'',$ids);
	$ids = str_replace('.txt','',$ids);
	$id = ($ids) ? $ids[count($ids)-1] + 1 : 1;

	debug($dbdir.$id.'.txt');

	file_put_contents($dbdir.$id.'.txt',$data);
	return $id;
}

function dbputcomment($id, $data) {
//this gets the full path to the file
	return file_put_contents($id,$data);
}

function dbdelcomment($id) {
	return unlink($id);
}

function commentdir($id, $mkdir = FALSE) {
	$dir = config('importdir').'comments/'.$id.'/';
	if ($mkdir)
		mkdir($dir);
	if (!is_dir($dir)) {
		debug("no '$dir'");
		return NULL;
	}
	return $dir;
}

function dbreaduser($userid) {

	if ($userid == AD_NAME)
		return array("userid" => AD_NAME,
			     "password" => AD_PASS,
			     "body" => "Administrator Account"
			    );
	return FALSE;
}

function dbputuser($uid, $record, $update = FALSE) {

# file mode only supports one user which is in the DEFINES file
	return FALSE;
}

function dbvisitor($cmd, $from = NULL, $data = NULL) {
	$dir = visitordir();
	if (!$dir) return 0;

	if ($cmd == 'range') {
		$c = 0;
# this is inefficient as it globs and comapres every file name each time
		foreach (glob($dir.'*') as $v) {
			$f = str_replace($dir,'',$v);
			$f = base64_decode($f);
			if (preg_match("/^[$from]+/",$f)) {
				$c++;
			}
		}
		return $c;
	}
	if ($cmd == 'lookup') {
		$r = array();
		foreach (glob($dir.'*') as $v) {
			$f = str_replace($dir,'',$v);
			$f = base64_decode($f);
			if (preg_match("/^[$from]+/",$f)) {
				$f = file_get_contents($v);
				$r[] = stringtorecord($f);
			}
		}
		return $r;
	}
	if ($cmd == 'read') {
		$f = urldecode($from);
		$f = base64_encode($f);
		debug("read: '$from', '$dir$f'");
		$f = file_get_contents($dir.$f);
		return stringtorecord($f);
	}
	return 0;
}

function dbreadvisitor($from) {
	$dir = visitordir();
	if (!$dir) return 0;

	$f = base64_encode($from);
	debug("read: '$from', '$dir$f'");
	if (is_file($dir.$f)) {
		$v = file_get_contents($dir.$f);
		return stringtorecord($v);
	}
	return 0;
}

function dbnewvisitor($from, $record) {
	$dir = visitordir();
	if (!$dir) return 0;

	$f = base64_encode($from);
	$data = recordtostring($record);
	debug("new: '$from', '$dir$f'");
	return file_put_contents($dir.$f,$data);
}

function dbputvisitor($from, $record) {
	$dir = visitordir();
	if (!$dir) return 0;

	$from = base64_encode($from);
	$data = recordtostring($record);
	debug($dir.$from);
	return file_put_contents($dir.$from,$data);
}

function dbdelvisitor($from) {
	$dir = visitordir();
	if (!$dir) return 0;

	$from = base64_encode($from);
	debug($dir.$from);
	return unlink($dir.$from);
}

function visitordir() {
	$dir = config('visitordir');
	if (empty($dir) || !is_dir($dir))
		return NULL;
	return $dir;
}
