<?php

/* DATA.PHP - misc. functions to load data */


/* datafile - reads an INI file that has data not key/value pairs in sections */

// $data = 1 means $file is data

function datafile($file, $data = 0) {

	if ($data == 0) {
		if (!($file = @file($file)))
			return array();
		$data = array();
	}
	else {
		$file = explode("\n",$file);
		$data = array();
	}

	$section = '';

	foreach ($file as $l) {
		$l = trim($l);
		if ($l == '' || $l[0] == ';')
			continue;

		if (preg_match('/^\[\s*(.*)\s*\]/',$l,$res)) {
			$section = $res[1];
			$data[$section] = '';
			continue;
		}

		if (!$section)
			continue;

		$data[$section] .= $l;
	}

	return $data;
}


/* configfile - read configuration data into an associative array */

// $file        file to parse or array of lines to parse
// $thisconfig  optional array to append to
// $quotes	remove " wrapped values if true (" " = space as a value)
// $test	return FALSE for missing ']' on a section, or a duplicate key 

// Returns: array() if file not found or file has no data
// This code is very forgiving and will always return an array. 
//
// This works just like typical INI parsing, with exceptions; quotes
// are never needed to wrap values, values can contain equal signs (=),
// if keys do not have equal signs and values it is assumed that a
// "section" is simply an array, if a section is blank ([]) increasing
// numbers are used, superglobals (within {}s) in values are evaluated.
// Single constants are preserved, "eol = PHP_EOL" -- but not 
// "err = E_ALL | E_STRICT".

// July, 31: some functionality has been removed from this code.

function configfile($file, $hasconfig = NULL, $quotes = FALSE, $test = 0) {

	if ($hasconfig)					// use existing array
		$config = $hasconfig;
	else
		$config = array();			// new array

	if (is_array($file)) {				// array passed
		$f = $file;				//  array cannot have
	}						//   newlines
	else {
		if (!is_file($file))
			return $config;
		$f = file($file,FILE_IGNORE_NEW_LINES|FILE_SKIP_EMPTY_LINES);
	}

	$line = 0;					// file line number
	$secount = 0;					// section counter
	$section = '';					// section name

	foreach ($f as $_) {
		$line++;
		$_ = trim($_);
		if ($_[0] == ';') continue;

		if ($_[0] == '[' && $_[strlen($_)-1] != ']') {
			$err = _configerr(2,$_,$line,$f);
			if ($test)
				return $err;
			continue;
		}
		if ($_[strlen($_)-1] == ']' && $_[0] != '[') {
			$err = _configerr(3,$_,$line,$f);
			if ($test)
				return $err;
			continue;
		}

		if (preg_match('/^\[(.*)\]/',$_,$res)) {
			$res[1] = trim($res[1]);

			// NOTE section can be int(0) if "[]" seen first
			$section = ($res[1]) ? $res[1] : $secount++;

			if (isset($config[$section]) && !is_array($config[$section])) {
				$err = _configerr(1,$section,$line,$f);
				unset($config[$section]);
				if ($test)
					return $err;
			}
			continue;
		}

		$aa = explode('=',$_);
		$noe = (count($aa) == 1);
		$a = array_shift($aa);
		$a = trim($a);
		if (is_empty($a))
			continue;

		$b = implode('=',$aa);		// $b can be '', let slide
		$b = str_replace('\n',"\n",$b);
		$b = trim($b);

		if ($quotes && preg_match('/^".*"$/',$b))
			$b = trim($b,'"');

		if (preg_match('/{\$[_A-Z]*\[/',$b)) {		// superglobal
			$b = '$b = "$b";';
			eval($b);
		}

		if (defined($b))				// constant
			$b = constant($b);

		if ($section !== '') {
			if (is_empty($b) && $noe)
				$config[$section][] = $a;
			else
				$config[$section][$a] = $b;
			continue;
		}

		$config[$a] = $b;
	}

	if ($test) return 'okay!';

	return $config;
}

function _configerr($err, $var, $line, $file) {

	$e[1] = "'$var' already defined; section overrides";
	$e[2] = "missing ']'";
	$e[3] = "missing '['";
	$s = "{$e[$err]}: '{$file[$line-1]}', line: $line";
	if (!function_exists('debug')) echo $s;
	else debug($s);
	return $s;
}


function parse_php($data) {

	$status = 'no error found';
	if (!preg_match('/^<\?php/',$data))
		return 'missing PHP open tag';

	$data = preg_replace('/^\s*<\?php/','',$data);
	$data = preg_replace('/\?>\s*$/','',$data);
	//$data .= "\nreturn 1;";

	if (eval($data) === FALSE) {
		$status = php_error($f,$l);
		$status .= " : line $l";
	}

	return $status;
}

function parse_php_file($file) {

	return parse_php(file_get_contents($file));
}
