<?php
// See http://gajennings.net/debug/
// Modified just for this to be slightly more efficient and to use config() for 
// debug options.

define('DEBUG_OPEN',"\n<div style='background:#fff; color:#000;'><pre>");
define('DEBUG_CLOSE','</pre></div>');
define('DEBUG_START','<b>message log:</b>');
define('DEBUG_EMPTY_STR',0);
define('DEBUG_EMPTY',1);

set_error_handler('error_handler');


/* debuglog - set the log level, dump status, register dump data */

function debuglog($log) {

	_debug('log',$log);
	if ($log && !_debug('reg')) {
		_debug('reg',1);
		register_shutdown_function('debug_msgs');
	}
}

/* debugdump - data to dump at the end */

function debugdump($dump) {

	_debug('dump',$dump);
	if ($dump && !_debug('dreg')) {
		_debug('dreg',1);
		register_shutdown_function('debug_dump');
	}
}

/* debug - add (or display) data/message */

function debug($msg = NULL, $arg = 0) {

	$log = _debug('log');
	if ($msg === NULL)
		return $log;
	if ($log == 0)
		return;

	if ($arg == -1) {		# not brilliant:
		$arg = 0;		#  need to reset this
		$log = -1;		#  and to temporarily set this
	}

	list($file,$line,$func) = caller(1);
	$func = caller(2,'function');

	if (($t = config('debugmod')) && !strstr($t,$file))
		return;
	if (($t = config('debugnomod')) && strstr($t,$file))
		return;

	if ($arg === 'type') {
		$msg = gettype($msg);
		$arg = 0;
	}
	else
	if ($msg === TRUE)
		$msg = '(true)';
	else
	if ($msg === FALSE)
		$msg = '(false)';
	else
	if ($msg === '' && DEBUG_EMPTY_STR)
		$msg = '("")';
	else
	if (is_resource($msg))
		$msg = 'Resource: '.get_resource_type($msg);
	else
	if (is_object($msg))
		$msg = 'Object: '.get_class($msg);
	else
	if (is_array($msg))
		$msg = 'Array: {'. flatten($msg) .'}';

	if ($arg == 0 && ($t = config('debugtruncate')))
		$arg = $t;

	if ($arg > 0 && strlen($msg) > $arg)
		$msg = substr($msg,0,$arg).' ...';

	if (!config('nodebugentities'))
		$msg = htmlentities($msg);

	if (config('debugconvertnl'))
		$msg = str_replace("\n",'\n',$msg);

	$c = '';
	if (config('debugcaller')) {
		list($f,$l,$na) = caller(2);
		$c = "($f,$l)";
	}

	if (config('debugnofunc'))
		$msg = "$c($file,$line) $msg";
	else
		$msg = "$c($file,$line,$func) $msg";

	if ($log == -1)
		print $msg . '<br>';
	else
	if ($log == 100)
		print '<!-- '.$msg.' -->'."\n";
	else
	if ($log == 200) {
		if (!isset($GLOBALS['debug_messages']))
			$GLOBALS['debug_messages'] =  '';
		$GLOBALS['debug_messages'] .= $msg.'<br>'."\n";
	}
	else
		_debug('msg',$msg);
}

/* _debug - debug initialize and configuration */

function _debug($key = NULL, $val = NULL) {
static $debug = array();

	if ($debug == array()) {
		$debug['log'] = 0;
		$debug['msgs'] = array();
		$debug['open'] = DEBUG_OPEN;
		$debug['close'] = DEBUG_CLOSE;
		$debug['start'] = DEBUG_START;
		$debug['empty'] = DEBUG_EMPTY;
	}

	if ($key === NULL)				// return array
		return $debug;

	if ($val !== NULL) {				// set a value
# special (internal) key 'msg':
		if ($key == 'msg')			// add a message
			return $debug['msgs'][] = $val;
		return $debug[$key] = $val;
	}

	if (isset($debug[$key]))			// get a value
		return $debug[$key];

	return '';
}


/* ERROR HANDLER */

/* error_handler - an example error handler */

function error_handler($errno, $errstr, $errfile, $errline) {

	$errfile = basename($errfile);
	$error = "$errstr ($errfile, $errline)";

	if ($errno == E_USER_ERROR)
        	exit($error);

	if (ini_get('html_errors'))
		$error = preg_replace('/\[.*\]/','',$error); 

	if (!function_exists('config')) {
		echo '<pre>This needs to be fixed:<br>';
		echo '<b>'.$error.'</b><br>';
		exit;
	}

	_debug('php_error',$error);		// save for later use

	$r = _debug('php_error_return');	// an odd option for testing
	if ($r !== TRUE && $r !== FALSE)
		$r = TRUE;

	if ($errno == E_NOTICE) {
		if (config('shownotices'))
			echo($error.'<br>');
		if (!config('nodebugnotices'))
			debug($error);
		return $r;
	}

	if (config('showerrors'))
		echo($error.'<br>');

	debug($error);

	return $r;
}


/* SHUTDOWN FUNCTIONS */

/* debug_msgs - display the message log (as a shutdown fuction) */

function debug_msgs() {

	restore_error_handler();		// no longer wanted
						//  in case error here (note 2)
	$log = _debug('log');
	if ($log <= 0 || $log >= 100)		// special cases
		return;

	$debug = _debug();
	if (count($debug['msgs']) == 0 && !$debug['empty'])
		return;

	print $debug['open'];
	print $debug['start'];
	foreach ($debug['msgs'] as $msg)
		print "\n$msg";
	print $debug['close'];
}

/* debug_dump - display various global data; see debuglog() */

// this can dump EVERYTHING, that is all data generated and used by this 
// code; no sensitive data is stored EXCEPT AS CONSTANTS WHICH CAN BE 
// DISPLAYED WITH A DEBUG SETTING - restricted to being logged in as Admin

function debug_dump() {

	if (($dd = config('dump')) == 0)
		return;
	$d = config('debug');

	// if debug set to display in HTML comments don't do two things:
	//  do not wrap output in the "<pre></pre>" tags
	//  do not htmlentitie-ize them and must eat '-->' and '--'

	print ($d == 100) ? "<!--\n" : _debug('open');

	ob_start(($d == 100) ? 'obcomments' : 'obentities');

## oops...
	foreach(explode(',',config('debugundump')) as $var)
		unset($GLOBALS[$var]);

	if ($dd == 1) {
		foreach(explode(',',config('debugdump')) as $var) {
			if ($var == 'html') dump(html(),'html ',0,0);
			if ($var == 'config') dump(config(),'config ',0,0);
			if ($var == 'rules') dump(_rules(),'rules ',0,0);
			if (isset($GLOBALS[$var]))
				dump($GLOBALS[$var],"$var ",0,0);
		}
	}

	if ($dd == 2) {
		dump($_GET,"_GET: ",0,0);
		dump($_POST,"_POST: ",0,0);
	}

	if ($dd == 3) {
		$defs = get_defined_constants(true);
		dump($defs['user'],"CONSTANTS ",0,0);
	}

	if ($dd == 4)
		dump($GLOBALS,"GLOBALS ",0,0);

	ob_end_flush();
	print ($d == 100) ? "\n-->" : _debug('close');
}


/* MISCELLANEOUS */

/* error - get some diagnostics */

// a supplemental support function - it might be cool to put this functionality 
// inside debug(), e.g. debug('','caller')

function error($msg = '', $arg = '') {

	switch ($msg) {
	case 'caller':
		// who called the caller
		list($file,$line,$func) = caller(2);
		$func = caller(3,'function');
		return "$file,$line,{$func}$arg";
	case 'trace':
		return backtrace();
	case 'backtrace':
		return backtrace(-1);
	case 'php_error':
		return _debug('php_error');
	}
}

/* backtrace - return a full backtrace array or a nice and compact string */

function backtrace($full = 0, $args = 1) {

	if ($full == -1) {
		if (!$args && defined('DEBUG_BACKTRACE_IGNORE_ARGS'))
			$bt = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
		else
			$bt = debug_backtrace();

		foreach ($bt as &$b)
			$b['file'] = basename($b['file']);

		return $bt;
	}
 
	ob_start();
	if (!$args && defined('DEBUG_BACKTRACE_IGNORE_ARGS'))
		debug_print_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
	else
		debug_print_backtrace();
	$out = ob_get_contents();
	ob_end_clean();

// see note 1
	$r = $_SERVER['DOCUMENT_ROOT'].dirname($_SERVER['PHP_SELF']).'/';
	if (PHP_EOL == "\r\n")
		$out = str_replace('\\','/',$out);
	$out = str_replace($r,'',$out);
	if (ini_get('html_errors'))
		$out = preg_replace('/ \[.*\]:/','',$out);

	if ($full == 1)
		return nl2br($out);

	$out = htmlentities($out);
	$out = explode("\n",$out);		// newlines in function args
						//  will interfere with explode
	array_shift($out);			// skip backtrace() backtrace
	foreach ($out as &$b)
		$b = substr($b,4);
	
	return implode('<br>',$out);
}

/* caller - get caller of caller (why is this function so big) */

function caller($depth = 1, $elem = NULL, $nb = 0, $rbt = 0) {

	if (defined('DEBUG_BACKTRACE_IGNORE_ARGS'))
		$bt = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
	else
		$bt = debug_backtrace();

	if ($rbt)
		return $bt;

	if ($elem) {
		$s = (isset($bt[$depth][$elem])) ? $bt[$depth][$elem] : '';
		if (!$nb && $elem == 'file')
			$s = basename($s);
		return $s;
	}

	if (!isset($bt[$depth]))
		return(array('?',$depth,''));

	$bt = $bt[$depth];
	if ($nb) 
		$file = (isset($bt['file'])) ? $bt['file'] : '';
	else
		$file = (isset($bt['file'])) ? basename($bt['file']) : '';
	$line = (isset($bt['line'])) ? $bt['line'] : '';
	$func = $bt['function'];

	if ($elem === '')
		return "$file,$line,$func";

	return array($file,$line,$func);
}

/* flatten - an implode() that handles associative arrays */

function flatten($a) {

	if (isset($a[0]) && !is_array($a[0]))
		return implode(',',$a);

	$s = '';
	foreach ($a as $k => $v) {
		if (is_array($v))
			$s .= "$k=".flatten($v);
		else
			$s .= "$k='$v',";
	}

	return rtrim($s,',');
}

/* dump - a variation of var_dump() */

// $s is a "title" (NULL to return output)
// $p is wrap in '<pre></pre>'
// $h is use htmlenties()

function dump($var, $s = '', $p = 1, $h = 1, $d = 0) {

	if ($p && $s !== NULL)
		print '<pre>';

	if ($s !== NULL)
		print $s;

	ob_start();

	if ($d == 0) {
		if (is_array($var))
			print_r($var);
		else
			var_dump($var);
	}
	else
	if ($d == 1)
		var_dump($var);
	else
	if ($d == 2)
		my_var_dump($var);
	else
		var_export($var);

	$out = ob_get_contents();
	ob_end_clean();

	if ($h)
		$out = htmlentities($out,ENT_NOQUOTES);

	if ($s === NULL)
		return $out;
	else
		echo $out;

	if ($p)
		print '</pre>';
}

// an anal version of var_dump()

function my_var_dump($var, $ret = 0) {

	ob_start();
	var_dump($var);
	$out = ob_get_contents();
	ob_end_clean();
	$out = str_replace("=>\n "," =>",$out);
	if ($ret)
		return $out;
	echo $out;
}


// OB callbacks

function obentities($buffer) {
	return htmlentities($buffer);
}

function obcomments($buffer) {
	return str_replace('--','-',str_replace('-->','',$buffer));
}

/*
NOTES
1. (Thought I would explain what happens next because for a second I was lost):
The PHP_EOL check is to fix Windows path separator, then the full path is 
reduced, and HTML error links (for function names) are removed. Then, (the 
default) is to remove the #numbers from the data (incorrect if more than 9 but 
deep but that is okay).
2. Found out the hard way.
*/
