<?php

/* GETPOST.PHP - GET/POST data parser/validator

   These functions try to reduce the complexity of parsing and validating 
   input data in various ways. There used to be tons of code here to try to 
   "automate" input parsing, doing both sanitation and filtering in a number 
   of different ways, someimes in pathetically odd ways. But it turns out that 
   minimal handling *is best done* here, and anything more than a single 
   filter (and length) to sanitize or validate *is best done* there--i.e. 
   where the data is used (and took a ton of code to prove that, so to speak.)

   Instead of trying to do everything at once and trying to anticipate needs, 
   as the previous versions did, it was found that starting with a "light 
   API", extending it to support needs as needs arose, [would have] made for 
   less coding and more logical code.
*/

// these are the input data "macros" (actually useful)
define('GET_','{foreach(explode(",",$get)as$_g){$$_g=(isset($_GET[$_g]))?trim($_GET[$_g]):"";}unset($_g);}');
define('POST_','{foreach(explode(",",$post)as$_p){$$_p=(isset($_POST[$_p]))?trim($_POST[$_p]):"";}unset($_p);}');

// usage:
// $post = 'name,button';
// eval(POST_);
// echo "name=$name, button=$button";


// first things first

foreach ($_POST as $name => $value) {
	$_POST[$name] = preg_replace("/\r/","",$value); // WINDOWS/MAC
	$_POST[$name] = trim($_POST[$name]);
	if (get_magic_quotes_gpc())
		$_POST[$name] = stripslashes($_POST[$name]);
}


/* getvar - read a $_GET variable (reduces redundancy) */

function getvar($var) {
	return (isset($_GET[$var])) ? $_GET[$var] : '';
}

/* getvars - read multiple $_GET variables */

function getvars($vars, $import = FALSE) {

	if (!is_array($vars))
		$vars = explode(',',$vars);

	foreach ($vars as $name) {
		$a[$name] = getvar($name);
		if ($import)
			$GLOBALS[$name] = $a[$name];
	}

	return $a;
}

/* postvar - read a $_POST variable */

// a slightly more complicated version 

function postvar($var, $filter = NULL, $len = 0) {

	if (!isset($_POST[$var]))
		return '';

	if ($filter) {
		if (function_exists($filter))
			$var = $filter($_POST[$var]);
		else
			$var = filter_input(INPUT_POST,$var,$filter);
	}
	else
		$var = $_POST[$var];

	if ($len > 0)
		$var = substr($var,0,$len);

	return $var;
}

/* postvars - read multiple $_POST inputs into an array */

# not complete as it does not allow for filter/length (will add when needed)...

function postvars($vars, $import = FALSE) {

	if (!is_array($vars))
		$vars = explode(',',$vars);

	foreach ($vars as $name) {
		$a[$name] = postvar($name);
		if ($import)
			$GLOBALS[$name] = $a[$name];
	}

	return $a;
}
