<?php
// This file is seperate from THIS - see http://gajennings.net/gmlp/
/*
   GMLP - The GMLP Markup Language Processor; version 0.9
   This code translates* any markup language to any other.**
   Copyright 2013-2014, Greg Jennings.
   License: The (PHP|MIT|Apache|Mozilla) License.
   See GMLP.TXT for the test text and the documentation.

*  I use the word "translate" but actually what happens is conversion 
   (or perhaps even more accurately substitution), and I interchange 
   the two words at times depending on context.
** Currently some output is "wrapped" in HTML P tags but that will 
   change.
*/

/* This code's initialization function is _gmlp() and is at end of file. */

/* gmlp_convert - convert string according to $gmlp_translate data */

// This converts a string according to the translation data; the string is 
// exploded on newlines and then treated as an array of lines; each line 
// will be wrapped in P tags, with some exceptions.

function gmlp_convert($data, $notrans = FALSE) {

	$data = explode("\n",$data);

	$p = 0;					// the one state flag
	$body = '';                     	// return value

	while (($_ = array_shift($data)) !== NULL) {
		if ($_ == '')
			continue;

		if ($notrans)
			goto skip;

// I don't particularly like using gotos, but their use does reduce
// complexity
# they are going to go away somehow...
		if (gmlp_skip($_))
			goto skip;

		if (gmlp_lines($_))
			goto skip2;

		if (gmlp_convert_block($data,$_))
			goto skip2;

		gmlp_inlines($_);

		if (_gmlp('entitytranslate'))
			gmlp_entities($_);

		gmlp_chars($_);
skip:
		if ($p == 0)
			$_ = '<p>'.$_;
		else
			$_ = '<br>'.$_;
		$p = 1;

// this is a trick "look into the future" thingy and is weird, but it works
		if (isset($data[0]) && $data[0] == '') {
			$_ .= '</p>';
skip2:			$p = 0;
		}

		$body .= $_."\n";
	}

	if ($p)
		$body .= '</p>';

	return $body;
}

/* gmlp_convert_str - do all the line conversions at once */

function gmlp_convert_str(&$_) {

	gmlp_inlines($_);

	if (_gmlp('entitytranslate'))
		gmlp_entities($_);

	gmlp_chars($_);
}

/* gmlp_skip - skip over lines that match a RE */

function gmlp_skip(&$_) {

	foreach (_gmlp('skip') as $s)
		if (preg_match($s,$_,$res)) {
			if (isset($res[1]))
				$_ = $res[1];
			return TRUE;
		}

	return FALSE;
}

/* gmlp_lines - REs to convert lines - skip other conversions */

function gmlp_lines(&$_) {

	foreach (_gmlp('lines') as $re => $repl)
		if (preg_match($re,$_,$res)) {
			if ($repl == '') {
				if (isset($res[1]))
					$_ = $res[1];
				return TRUE;
			}
			if (function_exists($repl)) {
				if (isset($res[1]))
					$_ = $res[1];
				$_ = $repl($_);
// the entire $res should be passed to the function; a change that will occur
				return TRUE;
			}
			if (isset($res[1]))
				gmlp_convert_str($_);
			$_ = preg_replace($re,$repl,$_);
			return TRUE;
		}

	return FALSE;
}

/* gmlp_inlines - apply REs/functions to lines */

function gmlp_inlines(&$_) {

	foreach (_gmlp('inlines') as $re => $repl) {
		if (function_exists($repl)) {
			if (preg_match_all($re,$_,$res))
				foreach ($res[0] as $k => $v)
					$_ = str_replace($v,$repl($res[1][$k]),$_);
		}
		else
		if (preg_match($re,$_))
			$_ = preg_replace($re,$repl,$_);
	}
}

/* gmlp_entities - string replacements on each line */

function gmlp_entities(&$_) {

	foreach (_gmlp('entities') as $str => $replace)
		$_ = str_replace($str,$replace,$_);
}

/* gmlp_chars - basic bold, italic, etc. */

function gmlp_chars(&$_) {

	foreach (_gmlp('chars') as $k => $v)
		$_ = _gmlp_chars($_,'\\'.$k,$v);
}

// every time I try to merge this function in above I lose a brain cell...

function _gmlp_chars($str, $char, $tag) {

	$repl = '/([^a-zA-Z=;\/])'.$char.'(.*)'.$char.'([^a-zA-Z\/\]])/U';
	$tags = '$1<'.$tag.'>$2</'.$tag.'>$3';
	$str = ' '.$str.' ';			// put in two spaces!
	$s = preg_replace($repl,$tags,$str);
	$s = substr_replace($s,'',-1);		// get out the spaces!
	$s = substr_replace($s,'',0,1);
	return $s;
}

/* gmlp_line - REs applied to each line of block conversions */

function gmlp_line(&$_) {

	foreach (_gmlp('line') as $re => $repl)
		if (preg_match($re,$_))
			$_ = preg_replace($re,$repl,$_);
}

/* gmlp_convert_block - convert lines between start and end REs */

// called with the input array starting one line past the start tag, 
// and the start tag line (both as references; the array will be 
// modified if block tag found, which is why array_shift is used 
// rather than foreeach)

function gmlp_convert_block(&$data, &$s) {

	if (!($block = _gmlp_lookup_block($s)))
		return FALSE;

	$end = $block['end'];
// these are a bit inefficient; the "load" code used to default them 
// all which I might just do again as it would increase performance
	$pre = (isset($block['pre'])) ? $block['pre'] : '';
	$post = (isset($block['post'])) ? $block['post'] : '';
	$first = (isset($block['first'])) ? $block['first'] : '';
	$last = (isset($block['last'])) ? $block['last'] : '';
	$repl = (isset($block['replace'])) ? $block['replace'] : '';
	$convert = (isset($block['convert'])) ? $block['convert'] : '';
	$nl = isset($block['newline']);

	if ($first) {
		if (!$convert)
			gmlp_line($s);
		else
			gmlp_convert_str($_);

		if (function_exists($repl))
			$s = $repl($s);

		$s = $pre.$s."\n";
	}
	else
		$s = $pre;

	$p = 0;
	while (($_ = array_shift($data)) !== NULL) {
		if (preg_match($end,$_))
			break;

		if (!$convert)
			gmlp_line($_);
		else
			gmlp_convert_str($_);

		if ($_ == '' && !$nl)
			continue;

		if (function_exists($repl))
			$_ = $repl($_);
		else
		if ($repl)
			$_ = $_.$repl;

		$s .= $_."\n";
	}

	if (isset($block['continue']))
		array_unshift($data,$_);

	if ($last === 0)			// eat last newline
		$s = substr_replace($s,'',-1);

	if ($last) {
		if (!$convert)
			gmlp_line($_);
		else
			gmlp_convert_str($_);
		$s .= $_;
	}

	if (function_exists($post))	// 'post' is either a function
		$s = $post($s);		//  or string to append
	else				//   might want to do more
		$s .= $post;

	return TRUE;
}

/* gmlp_open - read a translation data file */

// alternatively one can directly include function and/or definition files

function gmlp_open($definitions = '', $functions = '') {

	if ($definitions === '')
		$definitions = dirname(__FILE__).'/gmlp_def.php';
	if ($functions === '')
		$functions = dirname($definitions).'/gmlp_func.php';
	if ($functions) include $functions;
	include $definitions;
	return _gmlp($gmlp_translate);
}

/* gmlp_add - add data */

function gmlp_add($section, $data) {

	_gmlp($section,$data);
}

/* gmlp_add_block - add block data */

function gmlp_add_block($section, $data) {

	_gmlp($section,$data,TRUE);
}

/* internal functions */

// It just became a habit of mine to prepend an underscore on all functions that 
// are only used in one file, i.e. this function is just to support the 
// functions here and is not expected to be called by any other code.

/* _gmlp - initialize (on first call) then return data array */

// should be initialized with an array, see gmlp_open(); see also gmlp_add()
// if $var (data array member) is not defined return array() so that the 
// foreach blocks above "do nothing"

function _gmlp($var = '', $data = NULL, $block = FALSE) {
// note that this makes the code operate like a Class without being a Class
static $init = 0;
static $gmlp_translate = array();

	if (!$init) {

		if ($data) {
			echo("data passed to GMLP before init");
			return FALSE;
		}

		$init = 1;
		if (is_array($var))
			$gmlp_translate = $var;
		else
		if (is_file($var))
			include $var;
		else {
			debug("invalid '$var' passed to GMLP");
			return FALSE;
		}

		return TRUE;
	}

	// writing:
	if ($data !== NULL && !$block) {
		// check for setting non-array data first
		if (isset($gmlp_translate[$var]) && !is_array($gmlp_translate[$var])) {
			$gmlp_translate[$var] = $data;
			return;
		}
		foreach ($data as $key => $value)
			$gmlp_translate[$var][$key] = $value;
		return;
	}
	if ($data && $block) {
		foreach ($data as $key => $value)
			$gmlp_translate['blocks'][$var][$key] = $value;
		return;
	}

	// reading:
	if ($var === NULL)
		return $gmlp_translate;

	return (isset($gmlp_translate[$var])) ? $gmlp_translate[$var] : array();
}

/* index into data's 'blocks' section and return one if match */

function _gmlp_lookup_block($_) {

	foreach (_gmlp('blocks') as $b)
		if (preg_match($b['begin'],$_))
			return $b;
	return NULL;
}
