<?php

/* INC/INPUTS.PHP - parse (validate/filter/sanitize) POST data

   This provides a way parse POST data by defining an associative array to 
   instruct the code what to do with the data. It is the first step to true 
   re-usable code that can apply a variety of options to an unlimited 
   amount of input data.

   NOTE: This is a first draft. Full documentation is forthcoming.
*/

define('PREG_URL','/^(http[s]*:\/\/|)\w+\.\w+[\w\.\/~]*$/');
define('PREG_NAME','/[^[:word:]. ]/');

/* defines for result status if error - "map" to DISPLAY data for now */

define('BADFROM','NAMEBAD');
define('BADFROMLENGTH','LENGTH');
define('BADSITE','WEBSITE');
define('BADWORD','BADWORD');
define('BADDATALENGTH','CLENGTH');


/* createcomment - create a comment from POST data

   This processes the POST data - validate/filter/sanitize - creating a 
   record and returning a status value.
*/

function parse_inputs(&$record, $inputs) {
	if ($record === NULL)
		settype($record,'array');

	$res = '';
	foreach ($inputs as $name => $def) {
		$n = (isset($def['name'])) ? $def['name'] : $name;
		$err = parse_input($name,$def,$record[$n]);
		if ($err)
			$res = $err;
	}

	return $res;
}

/* parse_input - copy and optionally validate/sanitize an input */

function parse_input($name, $def, &$value) {

	$value = isset($_POST[$name]) ? $_POST[$name] : '';

	if (($r = $def['reject']) && $value) {
		if (function_exists($r) && $r($value))
			return $def['error'];
		else
		if ($value == $r)
			return $def['error'];
	}

	if (($f = $def['filter']) && $value) {
		if (function_exists($f))
			$value = $f($value);
		else {
			$t = preg_match($f,$value);
			// note1
			$value = htmlspecialchars($value);

			if ($r === 0)
				$t = !$t;
			if ($t)
				return $def['error'];
		}
	}

	// done here because filter can empty the value
	if ($value == '') {
		if ($d = $def['default']) {
			if (function_exists($d))
				$value = $d();
			else
				$value = $d;
		}
		return '';
	}
	if (($l = $def['length']) && strlen($value) > $l) {
		if (isset($def['lerror']))
			return $def['lerror'];
		$value = substr($value,0,$l);
	}

	return '';
}
/*
note1
The caller does not know how the data is parsed, so it cannot do the HTML 
entity conversion, as, for example, a filter function can do that (as in the 
comment data, by the function covert_user(), see below). So, currently, there 
is a constraint here that filter functions need to do it (which may change).
*/


/* comment_inputs - define input data for a comment */

function comment_input_def() {

/* associative array to define how to validate/default/sanitize the inputs */

	$def = array (
	  'date' => 
	  array (
	    'default' => 'configdatecmt',
	  ),
	  'from' => 
	  array (
	    'length' => '32',
	    'default' => 'nobody',
	    'reject' => 'nobody',
	    'filter' => '/[^[:word:]. ]/',
	    'error' => BADFROM,
	    'lerror' => BADFROMLENGTH,
	  ),
	  'title' =>
	  array (
	    'length' => '32',
	    'default' => 'nothing',
//	    'filter' => 'my_strip_tags',		// one way to do it
	    'filter' => '/[^[:word:].,\- ]/',
	    'error' => 'BADTITLE',
	  ),
	  'comment' =>
	  array (
	    'length' => config('postlimit'),
	    'reject' => 'checkbadinput',
	    'filter' => 'convert_user',
	    'error' => BADWORD,
	    'lerror' => BADDATALENGTH,
	    'name' => 'body',
	  ),
	);

	foreach ($def as &$d) {
		if (!isset($d['reject'])) $d['reject'] = '';
		if (!isset($d['filter'])) $d['filter'] = '';
		if (!isset($d['length'])) $d['length'] = '';
	}

	return $def;
}

/* Next version will be more like: */

/*
define('BADFROM',1);
define('BADFROMLENGTH',2);
define('BADSITE',3);
define('BADWORD',4);
define('BADDATALENGTH',5);
define('BADEMPTY',6);
*/

/*

; This is INI encoding, which may happen so to be easily edited.

[from]
length = 32
default = anonymous
reject = aonymous
filter = "/[^[:word:]. ]/"
error = BADFROM
lerror = BADFROMLENGTH

[date]
default = datestr

[title]
filter = "/^(http[s]*:\/\/|)\w+\.\w+[\w\.\/~]*$/"
error = BADSITE

[comment]
length = 3000
reject = checkbadinput
filter = convertdata
error = BADWORD
lerror = BADDATALENGTH
*/

