<?php
/*
	"...we know that the initialization of a program can't be written 
	until the program itself takes shape."
	  -- Donald Knuth
*/

/* CONFIG.PHP - This configures the core of the application.

This contains all "sitewide" configuration data as an associative array as a 
static variable in the config() function that is initialized on first call. 
All data access is through that function (with a few "helper" functions).

The odd things here are how "sections" are read (and we need more documentation 
about that), how 'debug' is handled and how $arg is checked.

It may be useful to think of config() as a super global $_CONFIG and that the 
function config() just accesses that array. In fact, we could do away with the 
static $config array and just use $GLOBALS['config'] as an array -- which if 
we had thought of sooner we would have done just that. (And, once the Admin 
configuration editor is finished all this code will disappear.)

All this code does is create a static array of configuration data and provide 
a function to access that data.
*/


/* the initialization function */

// config() initializes itself on first call, storing data in a static array

function _config_config() {
	config();
}


/* config - read/write configuration data */

// note1

function config($var = NULL, $val = NULL) {
static $config = array();

	if ($config == array()) {

		if (!is_file($f='config.ini'))
			user_error("'$f' was not found",E_USER_ERROR);
		$config = parse_ini_file($f,TRUE);
		if ($config == FALSE)
			user_error("cannot continue with error in '$f'",E_USER_ERROR);

		if (is_file($f='sections.ini')) {
			$sections = parse_ini_file($f,TRUE);
			if ($sections == FALSE)
				user_error("cannot continue with error in '$f'",E_USER_ERROR);
		}

		$t = ($t = $config['timezone']) ? $t : 'America/New_York';
		date_default_timezone_set($t);

		// if no sections file make the default

		if (!isset($sections)) {
			$sections['name'] = $config['title'];
			$sections['subtitle'] = $config['subtitle'];
			$sections['description'] = $config['description'];
		}

		// move sections data into $config['sections']; the 
		// default section goes into $config['sections']['']

		foreach ($sections as $k => $v)
			if (!is_array($v)) {
				$config['sections'][''][$k] = $v;
				unset($sections[$k]);
			}

		foreach ($sections as $k => $v)
			$config['sections'][$k] = $v;

		$config['local'] = 0;
		$config['version'] = $GLOBALS['VERSION'];
		if (stristr($GLOBALS['VERSION'],'admin'))
			$config['admin'] = 1;
		if ($_SERVER['SERVER_NAME'] == 'localhost')
			$config['local'] = 1;

// sets 'debug' only if logged in as Admin or running Admin, but with a 
// special value of 100 even if not logged in as Admin (to debug Admin code 
// for example; see file DEBUGGING); need to do this early

		if ($config['debug'] == 100)
			debuglog(100);
		else
		if (userstate() || isset($config['local']) || isset($config['admin'])) {
			if (($t = getvar('debug')) !== "")
				$config['debug'] = $t;
			if ($config['debug'])
				debuglog($config['debug']);
			if ($config['dump'])
				debugdump($config['dump']);
		}

		// make sure directory variables end appropriately

		foreach($config as $k => $v)
			if ($v && !is_array($v) && preg_match('/dir$/',$k))
				$config[$k] = rtrim($v,'/') . '/';

		// create some miscellaneous data

		$config['ip'] = $_SERVER['REMOTE_ADDR'];
		$config['ua'] = $_SERVER['HTTP_USER_AGENT'];
		$config['basedir'] = dirname($_SERVER['PHP_SELF']);
		if ($config['basedir'] != '/') $config['basedir'] .= '/';
		$config['siteurl'] = 'http://'.$_SERVER['HTTP_HOST'];
		$config['selfurl'] = $config['siteurl'].$_SERVER['PHP_SELF'];
		$config['siteurl'] = $config['siteurl'].$config['basedir'];

// unlike other GET inputs, 'arg' is checked here as it is needed to set 
// some configuration variables (see also RULES.PHP)...

		$arg = getvar('arg');
		if (!isset($config['sections'][$arg])) {
			if (isset($config['argerrors']))
				notfound();
			$_GET['arg'] = '';
			$arg = '';
		}

		// stash current section name for MySQL and files access

		$config['arg'] = ($arg) ? $arg : 'root';
		$config['pagedir'] .= $config['arg'].'/';
		$config['importdir'] .= $config['arg'].'/';

// this overrides settings by section (some settings won't make sense in a 
// section; see file SECTIONS); first make a copy of current section; this 
// also will allow a section to change 'debug' if it was already set
# yeah, not too cool...
		$debug = $config['debug'];
		$config['section'] = $config['sections'][$arg];
		$section =& $config['section'];
		foreach ($section as $name => $value)
			if (!in_array($name,array('subtitle','hidden')))
				$config[$name] = $value;
		if ($debug && $debug !== $config['debug'])
			debuglog($config['debug']);

		// Admin overrides

		if (isset($config['admin']) && isset($config['config.admin'])) {
			foreach ($config['config.admin'] as $name => $value)
				$config[$name] = $value;
		}

		// have to do this again just in case a dir was set in section

		foreach($config as $k => $v)
			if ($v && !is_array($v) && preg_match('/dir$/',$k))
				$config[$k] = rtrim($v,'/') . '/';

		if (config('allowbasecomments'))	// to avoid a conflict
			$config['allowcomments'] = 0;

		debug('cookies: '.cookie_str());
	}

	if ($var === NULL)
		return $config;

	if ($val !== NULL)
		return $config[$var] = $val;

	if (isset($config[$var]))
		return $config[$var];

	return '';
}


// a few "support" functions - these simply reduce some inline string literals 

/* htmdir - return html directory with possible filename */

function htmdir($file = '') {

	return config('htmdir').$file;
}

/* themedir - return theme directory with possible filename */

function themedir($file = '') {

	return config('htmdir').config('themedir').$file;
}

function imgdir($file = '') {

	return config('imgdir').$file;
}

function configdate($time = '') {

	if ($time == '') $time = time();
	return date(config('datestr'),$time);
}

function configdatetime($time = '') {

	if ($time == '') $time = time();
	return date($d=config('datetime')?$d:DATE_RFC3339,$time);
}

function configdatecmt($time = '') {

	if ($time == '') $time = time();
	return date(config('datestrcmt'),$time);
}

/*
note1
What is most interesting about config() is that the result is an associative 
array that defines the site. And, once the site is configured and set up Admin 
can be used to var_export() this array. Then, that output can be copied and 
pasted in over this code (with a little work) in fact, the next version WILL 
allow that to be done. RULES.PHP and MOD/HTML.PHP also use a static array for 
data.
*/
