<?php

/* DISPLAY.PHP - Functions to display data.

   Most functions here are called by a process we call RULES and more 
   information can be found in RULES.PHP.

   This code is the heart of THIS as it does all the data input handling 
   and all the HTML display -- hence the name. Everything goes through this 
   code (so to speak).

   This code has it's own Web Template functionality, that is, a single-quoted 
   PHP string can contain a PHP variable name: '<title>$title</title>'. All the 
   HTML templates work this way -- two levels deep, i.e. one template can be 
   within another template.
*/


/* module initialization function */

function _config_display() {

	debug('gmlp');
	//echo(parse_php_file('translate.php'));	// only to find
	//echo(parse_php_file('transfuncs.php'));	//  syntax errors
	gmlp_open('translate.php','transfuncs.php');
	gmlp_add('entitytranslate',config('entitytranslate'));
}


/* display - print arguments/template */

function display() {
	$args = func_get_args();
	if ($html = html($args[0]))
		print $html;
	else
		print implode(' ',$args);
}

/* displayhtml - display content/HTML mixed with PHP variables */

function displayhtml() {

	$args = func_get_args();
	$templ = $args[0];

	// set variable variable if there is one

	$file = 0;				// flag for template is file
	if (isset($args[1])) {
		if ($args[1] == 'file')		// special case
			$file = 1;
		else {
			$val = str_replace('$','',$args[1]);
			$$val = $args[2];	// assumes if 1 then 2
		}
	}

	displaycalltheme(__FUNCTION__,$templ);	// tell theme "we are here"

	html('template',$templ);
	$html = html();				// data that can be referenced 
	$config = config();			//  by the HTML
	$arg = getvar('arg');

	$text = htmltempl($templ,$file);	// silently fail if not found
	if (!$text)
		return;

	// if data passed was a string it might reference variables
	// and the data passed is evaled and assigned to variable name given

	if (isset($val) && is_string($$val)) {
		if (eval(_textset($val,$$val)) === FALSE)
			_showerror();
	}

	// and the HTML might reference variables
	// (this is what allows a template to include a template)

	// note1

	if (!isset($val)) {
		if (eval(_textset('text',$text)) === FALSE)
			$err = _showerror();
	}

	// and now the HTML and all it's variables are displayed

	if (isset($direct) || htmldirect($templ))    // not all need next eval
		print $text;
	else
	if (!isset($err)) {
		if (eval(_textset('text',$text)) === FALSE)
			_showerror($text);
		else
			print $text;
	}
}

/* displayentries - display posts */

function displayentries() {

	$ids = dblistrecords();			// array if IDs
	if ($ids === FALSE) {
		$msg = geterror('NOTREAD');
		if (debug()) $msg .= ' '.dberror();
		displaymessage('Error',$msg);
		return;
	}

	sortrecords($ids);			// config('sortfunc')

	$arg = getvar('arg');
	$start = (int)getvar('start');
	if ($start < 0 || $start >= count($ids))
		$start = 0;

	$i = $c = 0;
	$end = config('postsperpage');
	foreach ($ids as $id) {
		if ($i++ < $start)
			continue;

		$record = readrecord($id);
		if (!$record)			// should never happen but 
			continue;

		if (isset($record['draft'])) {
			debug('draft skipped');
			$i--;
			continue;
		}

// record goes "out to User Agent" through this:

		displayrecord($record,$id);

		if (++$c == $end)
			break;
	}

	if ($c == 0)
		displaymessage('No Posts','This section has no posts yet.');

# "more" and/or "prev" links (STILL really sloppy and will be replaced)
	if (($c > 1 || $start > 0) && !config('allowbasecomments') && htmltempl('moreprev')) {
		$start = $start - config('postsperpage');
		html('count',$i);
		html('start',$start);
		html('more_vis',($i < count($ids)) ? 'visible' : 'hidden');
		html('prev_vis',($start >= 0) ? 'visible' : 'hidden');
		displayhtml('moreprev');
	}

	if (config('allowbasecomments'))
		displaycomments();
}

/* displayentry - display a single entry (called when 'post'ing) */

// see RULES.PHP for input validation

function displayentry() {

	$id = getvar('id');
	if ($id === '' || !dblistrecords($id)) {
		displayhtml('postnotfound');
		return;
	}

	$record = readrecord($id);
	displayrecord($record,$id);

	if (config('allowcomments') && !isset($record['nocomments']))
		displaycomments();
}

/* displaycomments - display comments if there are any */

// comments are associated with an entry; just display them all, we do not
// expect to be a site with thousands of comments see config('maxmsgs')

function displaycomments() {

	$entryid = (int)getvar('id');		// ('' becomes 0)

	debug("'$entryid'");

	$ids = dblistcomments($entryid);
	if ($ids === FALSE)                     // db query error
		return;

	if (!$ids && config('nocommentadd'))
		return;

	displayhtml('commentsintro');

	$msgcount = count($ids);

# example of "convoluted code" and it ain't pretty...

	if ($msgcount) {
		if (config('rsortcomments'))
			$ids = array_reverse($ids);

		foreach ($ids as $id) {
			$record = dbreadcomment($id);
			commentadjust($record);
			displayhtml('comment','$record',$record);
		}
	}
	else
		displayhtml('nocomments');

	if (config('nocommentadd')) {
		displayhtml('commentsclosed');
		return;
	}

	if ($msgcount >= config('maxmsgs'))
		displayhtml('commentsmax');
	else
		displaycommentform();
}

/* displaycommentform - displays 'commentform' template */

// note2

function displaycommentform() {

	$html = html();
	$arg = getvar('arg');
	$entryid = getvar('id');
	$url = make_url('submit');
	$from = cookie_get('from');

// this function modifies the $html data through it's function, html()

	if (config('commentwebsite'))
		html('titlelink',$html['weblink']);
	if (!config('visitorcode'))
		html('hidecode','hide');

	if ($from) {
		$_POST['from'] = $from;
		if (config('fromlogout')) {
			html('hidefrom','hide');
			$link = str_replace('$1',$from,$html['frominlink']);
			html('fromlink',$link.' '.$html['logoutlink']);
		}
	}

	html('formurl',$url.'#comments');
	html('formcancel',isset($_POST['button']) ? '' : 'disabled');
	html('formsubmit','');

	if (config('mustpreview')) {
		html('formsubmit',isset($_POST['button']) ? '' : 'disabled');
		if (htmlgeterror())
			html('formsubmit','disabled');
		if (!isset($_POST['comment']) || is_empty($_POST['comment']))
			html('formsubmit','disabled');
	}

	foreach ($_POST as &$p)
		$p = htmlentities($p);

	displayhtml('commentform');
}

/* displaysubmit - result of form to submit user comment */

// (as long as $code is empty, as it should be if 'visitorcode' is undefined
// or zero, all of the "visitor" functions do nothing; see file VISITORS)
# this function needs one more reshaping

function displaysubmit() {

	$id = getvar('id');
	$button = postvar('button');

	if ($id === '' || !dblistrecords($id)) {	// ID error? cancel
		$id = 0;
		$button = 'cancel';
	}

	if ($id == 0)
		$url = make_url();
	else
		$url = make_url('post');

	if ($button == 'cancel')
		redirect($url);	
// $url retains "#comment" from the form; it gets lost if 'noredirect = 1'

	$status = createcomment($record);

	$code = postvar('code');				// ignored if
	$vcode = visitor_validate($record['from'],$code);	//  not enabled

	// now we offer up some feedback if any inputs are invalid

	if (empty($record['body'])) 
		$error = htmlseterror('SUBMIT');

	if ($status)
		$error = htmlseterror($status);

	if ($vcode === FALSE)
		$error = htmlseterror('FROM');

	if ($vcode === NULL)
		$error = htmlseterror('CODE');

	if ($button == 'preview' || isset($error)) {
		rules_run('preview');			// call another rule
		exit;					//  (it's okay)
	}

	if ($button == 'submit' && !isset($error)) {

		visitor_store($record['from'],$code);
		recorduserdata($record);		// adds stuff for Admin
		$r = dbwritecomment($id,$record);
		$url .= "#$r";
	}

	redirect($url);
}

/* displaypreview - preview a comment */

function displaypreview() {

	$status = createcomment($record);		// from POST data
	if ($status)
		$error = htmlseterror($status);
	displayhtml('commentpreview');
	commentadjust($record);
	displayhtml('comment','$record',$record);
}

/* createcomment - create a comment from POST data */

function createcomment(&$record) {

	$inputs = comment_input_def();		// see INC/INPUTS.PHP

	// note3
	if (config('commentwebsite')) {
		unset($inputs['title']);
		$inputs['title']['filter'] = PREG_URL;
		$inputs['title']['error'] = BADSITE;
		$inputs['title']['reject'] = 0;
	}
	$status = parse_inputs($record,$inputs);
	return $status;
}


/* displayrecord - display record (post) data */

function displayrecord($record, $id = '') {

	$op = getvar('op');
	$arg = getvar('arg');

	$single = ($op == 'post');

	if ($op == 'page')
		$record['post'] = make_url('page',$id);
	else
		$record['post'] = make_url('post',$id);

	debug($record);

	if ($single) {
		$htfile = 'entry';
		hasmore($record['body'],0);
	}
	else {
		$htfile = 'entries';
		if (isset($record['noview']))
			$htfile = 'entries_n';

		$record['curl'] = $record['murl'] = '';
		if (config('allowcomments') && !isset($record['nocomments']))
			$record['curl'] = commentlink($record['post'],$record);

		if (hasmore($record['body']))
			$record['murl'] = morelink($record['post'],$record);
	}

	// usercode stuff; might modify record (see file USERCODE)

	if (!config('nousercode') && function_exists('usercode'))
		usercode($record);

// goes out to UA here:

	displayhtml($htfile,'$record',$record);
}


/* displaypage - display a "page" */

function displaypage($id = '') {

	if (empty($id))
		$id = getvar('id');
	$arg = getvar('arg');

	if (!ctype_alnum($id))
		$id = '';

	if ($id === '' || !dblistpages($id)) {
		displaymessage('pagenotfound');
		return;
	}

	$record = readfilerecord($id);			// MOD/RECORD.PHP
	$record['post'] =  make_url('page',$id);
	displayhtml('page','$record',$record);
}


/* displayhelp - display site help page */

// this is a "second draft" (and still lame)

function displayhelp() {

	debuglog(0);

	$help = config('helpdir').'index.html';
	if (!is_file($help)) {
		display('helpnotfound');
	}
	else {
		html('code',config('visitorcode') ? '' : 'hide');
		displayhtml($help,'file');
	}
}


/* displayfile - display a file */

function displayfile($view = '') {

	$srcdir = config('srcdir');
	$file = getvar('id');

	if (strpos($id,'..') !== FALSE || !is_file($srcdir.$id))
		displayerror('filenotfound');			// exits

	preg_match('/\.([^.]+)$/',$file,$res);
	$typ = (isset($res[1])) ? $res[1] : 'txt';

# there is probably some way/place/thing that can lookup mime types...
	switch ($typ) {
	case 'htm':
	case 'html':
		$type = 'text/html';
		break;
	case 'txt':
	case 'ini':
	case 'xml':
	case 'php':
		$type = 'text/plain';
		break;
	case 'zip':
		$type = 'application/zip';
		break;
	case 'jpeg':
	case 'jpg':
		$type = 'image/jpeg';
		break;
	case 'png':
		$type = 'image/png';
		break;
	default:
		echo 'file type unsupported, sorry';
		return;
	}

	if ($view) {
		if (strpos($type,'text') !== 0)
			echo 'file type unsupported';
		else {
			debuglog(0);
			header('Content-Type: '.$type);
			readfile($srcdir.$file);
		}
		return;
	}

	$size = filesize($srcdir.$file);

	debuglog(0);
	header('Content-Type: '.$type);
	header('Content-Length: '.$size);
	header('Content-Disposition: attachment; filename="'.$file.'"');
	readfile($srcdir.$file);
}


/* displaymessage - display a message in place of a post */

function displaymessage($title, $message = '') {

	debug($title);
	if (config('nodisplaymessages'))
		return;

	if ($html = html($title)) {			// $title defined?
		if (is_string($html)) {			
			displayhtml($title);		// template name
			return;
		}
		else {
			$title = $html['title'];	// array of two strings
			$message = $html['message'];
		}
	}

	$error['title'] = $title;
	$error['message'] = $message;
	displayhtml('error','$error',$error);
}

/* displayerror - display generic (fatal) message (page) and stop */

function displayerror($title, $message = '') {

	if ($html = html($title)) {			// $title defined?
		if (is_string($html)) {			
			displayhtml($title);		// template name
			return;
		}
		else {
			$title = $html['title'];	// array of two strings
			$message = $html['message'];
		}
	}

	html('title',$title);
	html('message',$message);
	displayhtml(html('template.html'),'file');
	// 'template.html' is in HTM/HTML.INI (and defaults to template.html)
	exit;
}


/* commentadjust - changes comment "title" to a website link */

// note3

function commentadjust(&$record) {
	if (!config('commentwebsite')) return;

	if ($url = $record['title']) {
		// since can be turned on/off, guard against turned on 
		//  for existing comments
		if (preg_match(PREG_URL,$url)) {
			if (strpos($url,'http://') !== 0)
				$url = 'http://'.$url;
			$record['from'] = "<a href=\"$url\">{$record['from']}</a>";
		}
		$record['title'] = '';
	}
}


/* commentlink - form the "comments" link (curl) */

function commentlink($url, $record) {

	$n = dbcountcomments($record['id']);
	if ($n === FALSE)
		return '';

	$link = ($t = html('commenttext')) ? $t : "comments ($n)";
	$link = str_replace('$N',$n,$link);

	$link = "<a href='$url#comments'>$link</a>";

	if (($linkwrap = html('commentlink')) != "")
		$link = str_replace('$1',$link,$linkwrap);

	return $link;
}

/* morelink - the "there is more" post link */

function morelink($url, $record) {

	if (isset($record['more']) && $record['more'])
		$more = $record['more'];
	else
		$more = ($m = html('moretext')) ? $m : "there is more";
	$link = "<a href='$url#more'>$more</a>";
	return $link;
}

/* hasmore - make the post have a "#more" tag or truncate at tag */

function hasmore(&$body, $trunc = 1) {

	if ($trunc) {
		$m = strpos($body,'<!--more-->');
		if ($m !== FALSE)
			$body = substr($body,0,$m);
		return $m;
	}
	$body = str_replace('<!--more-->','<span id="more"></span>',$body);
}


// the displayhtml() support functions

/* _textset - see how it is called */

function _textset($name, $text) {

	$text = _addslashes($text,'"');
	return "\$$name = \"$text\";";
}

function _addslashes($text) {

	if (config('escapebackslash'))
		$text = addcslashes($text,'"\\');
	else {
		// this prevents error if the string ends in an odd 
		//  number of backslashes
		$text = addcslashes($text,'"');
		if (preg_match('/([\\\\]+)$/',$text,$res)) {
			$l = strlen($res[1]);
			if (($l & 1) == 1)
				$text .= '\\';
		}
	}
	return $text;
}

/* _showerror - display the PHP error message that failed eval */

function _showerror($text = NULL) {

	$message = php_error($file,$line);
	$errline = '';
	if ($text) {
		$text = explode("\n",$text);
		$errline = htmlentities($text[$line-1]);
		$errline = " line $line:<br>$errline</tt>";
	}
	$templ = html('template');
	$message = "$message<br><tt>\$html['$templ']$errline";
	if (config('showerrors'))
		displaymessage('Template error',$message);

	return 1;
}


/* displaycalltheme - call theme with function name, argument */

function displaycalltheme($function, $data) {

	if (function_exists('theme_function'))
		theme_function($function,$data);
}

/*
note1. Any template that has $_POST references in it, needs to be a "first 
level" template only (explained in detail in TEMPLATES.TXT), otherwise PHP 
variable text ($foo) will evaluated - we 'direct' the commentform template to 
prevent that. To gaurd against mistakes in the naming, this can be used:
	if (strpos($text,'$_POST') !== FALSE)
		$direct = TRUE;
Obviously, if anyone thinks anything EVEN NEAR allowing $_POST in a 
template is stupid they should not use this code.
note2. The four functions (displaycommentform(), displaysubmit(), 
displaypreview(), createcomment()) are all the functions that process visitor 
comment data, they: display the FORM which may contain comment data; handle the 
data when SUBMIT button pressed; preview the data when error or if PREVIEW 
button pressed; read and parse the submitted data. The last is the pivot point. 
note3. This is an example of data overloading (I guess it could be called), one 
form can have the "title" input as a subject, and the other a website, instead 
of having two templates we can modify the data. It is kind of odd, but it is 
interesting -- next version will be different though.
*/
