<?php

/* MOD/PAGES.PHP - This is to read/write records as files in the 'pagedir'.

The use of 'db' for the function names is only to be similar to the other, 
related functions to read/write MYSQL records.

A version using MySQL will be created (eventually). It might then be merged 
with the post API (mysql.php and mod/record.php) as it shares the content 
format. (This could be made more content repository like.)

See also readfilerecord() in module RECORD.
*/

function makepagesdir() {

	$dir = config('pagedir');
	return mkdir($dir);
}

function dblistpages($id = NULL, $dir = NULL) {

	if (!$dir)
		$dir = config('pagedir');

	if (!is_dir($dir))
		return FALSE;

	if ($id)
		return (is_file($dir.$id.'.txt')) ? $id : 0;

	$ids = glob($dir.'*.txt');
	foreach ($ids as $k => $v) {
		$v = str_replace($dir,'',$v);
		$ids[$k] = str_replace('.txt','',$v);
	}
	natsort($ids);
	return $ids;	// will be array() if no files found
}

function dbreadpage($id, $dir = NULL) {

	if (!$dir)
		$dir = config('pagedir');

	if (!($rec = file_get_contents($dir.$id.'.txt')))
		return $rec;

	$record = stringtorecord($rec);
	$record['id'] = $id;
	return $record;
}

function dbgetpage($id, $dir = NULL) {

	if (!$dir)
		$dir = config('pagedir');

	return file_get_contents($dir.$id.'.txt');
}

function dbgetpageheaders($id, $dir = NULL) {

	if (!$dir)
		$dir = config('pagedir');

	$fd = fopen($dir.$id.'.txt','r');
	if (!$fd)
		return FALSE;

# kind of a code duplication of MOD/RECORD.PHP, stringtorecord()
	while ($s = fgets($fd)) {
		if ($s == "\n") break;
		$headers[] = $s;
	}
	fclose($fd);

	foreach ($headers as $_) {
		$h = explode(':',$_);
		$name = array_shift($h);		//  tag: text
		$name = strtolower($name);		//  tag:
		$value = implode(':',$h);		//  tag: Title:Foobar
		$record[$name] = trim($value);
	}

	return $record;
}

function dbnewpage($data, $id = '', $dir = NULL) {

	if (!$dir)
		$dir = config('pagedir');

	if ($id === '') {
		$files = glob($dir.'*.txt');
		natsort($files);
		foreach ($files as $f) {
			$f = str_replace('.txt','',$f);
			$f = str_replace($dir,'',$f);
			if (is_numeric($f))
				$id = (int)$f;
		}
		$id++;
	}

	$data = _check_page($data);
	return dbputpage($id,$data);
}

function dbputpage($id, $data, $dir = NULL) {

	if (!$dir)
		$dir = config('pagedir');

	if (strpos($id,'.txt') === FALSE)
		$id = $id.'.txt';

	$data = _check_page($data);
	return file_put_contents($dir.$id,$data);
}

function dbdelpage($id, $dir = NULL) {

	if (!$dir)
		$dir = config('pagedir');

	return unlink($dir.$id.'.txt');
}

function _check_page($data) {

	$record = stringtorecord($data);

	if (!isset($record['title'])) {
		$record['title'] = 'Title';
		if (!isset($record['date']))
			$record['date'] = date(config('datestr'));
		return recordtostring($record);
	}

	return $data;
}
