<?php

/* USERS.PHP - handles user login/logout via cookies

This is almost a standalone API and can be by moving a few things around...

Currently there is only one pre-defined user, 'admin', as THIS was designed to
be a single-user website -- the capability exists for this to be changed.

In File Mode this is not used.

There is a really simple database (see file DATABASE):

	+-----------------------------+
	| userid                      | VARCHAR UNIQUE
	+-----------------------------+
	| pasword: p6gYt5as1          | TEXT
	| name: Jones!                |
	|                             |
	| I am a great person.        |
	+-----------------------------+

The following functions are defined:

	userstate()
	userlogin($userid, $password)
	userlogout()
	usererror($user, $userid)
	usernew($userid, $passwd, $body = '')
	userget($userid)
	userput($userid, $passwd, $body = '', $append = 0)
	userdel($userid)
	thiscrypt()
*/

define('USER_ID','[a-zA-Z0-9_]+');


/* userstate - returns user ID string if user logged in else empty string */

// there are two cookies, userid and userdat; userdat is encoded userid hash
// if hash is wrong nothing happens - use of userdat solely prevents someone
// creating a cookie by editing a browser cookie file - if we had just that
// one userid cookie that could let someone in - if our password salt got
// leaked someone could create a userdat cookie as well - we will add more
// data to userdat at some point...

// Yeah know, we might be able to combine the two cookies into one...

function userstate() {

	$userid = cookie_get('userid');
	debug("userid '$userid'");

	if ($userid) {
		$c = cookie_get('userdat');
		$c = cookie_decode($c,$userid,$_SERVER['REMOTE_ADDR']);
		if ($c != $userid) {
			debug("userdat '$c' != '$userid'");
			return '';
		}
	}
	return ($userid) ? $userid : '';	// NULL, FALSE become ''
}


/* userlogin - check user exists; set cookie if yes, returns user RECORD */

function userlogin($userid, $password) {

	debug("'$userid'");

	$userid = trim($userid);
	$userid = strtolower($userid);

	$record = userget($userid);
	if (!$record) {
		debug(usererror($record,$userid));
		return FALSE;
	}

	$passwd = $record['password'];
	if (thiscrypt($password,$passwd) != $passwd) {
		debug("user '$userid' password wrong");
		return 0;
	}

	cookie_set('userid',$userid);
	$c = cookie_encode($userid,$_SERVER['REMOTE_ADDR']);
	cookie_set('userdat',$c);

	debug("userid '$userid' userdat '$c'");

	return $record;
}


/* userlogout - forget user information */

function userlogout() {

	cookie_unset('userid');
	cookie_unset('userdat');
# more here later
}


/* usererror - return error string */

function usererror($user, $userid) {

	if ($user === "") $e = "is empty";
	if ($user === NULL) $e = "is invalid";
	if ($user === FALSE) $e = "not found";
	if ($user === 0) $e = "password wrong";

	return "user '$userid' $e";
}

/* userid - check user id */

function userid($userid) {

	debug("'$userid'");

	preg_match('/('.USER_ID.')/',$userid,$res);
	if (isset($res[1]) && $res[1] != $userid)
		return FALSE;
	return TRUE;
}

/* usernew - add a new user to the database */

// password is plain text and gets encrypted here

function usernew($userid, $passwd, $body = '') {

	debug("'$userid','$passwd'");

	if (userget($userid))				// already exists?
		return 0;

	$record['name'] = $userid;
	$record['password'] = thiscrypt($passwd);
	$record['body'] = $body;

	if (!dbnewuser($userid,$record))
		return NULL;

	return $record;
}


/* userget - read user as a 'record' */

function userget($userid) {

	debug("'$userid'");

	if (!$userid) return '';
	if (preg_match("/\W/",$userid)) return NULL;

	return dbreaduser($userid);
}


/* userput - update user data */

// password is plain text and encryped with SALT1

function userput($userid, $passwd, $body = '', $append = 0) {

	debug("'$userid','$passwd'");

	if (!($record = userget($userid)))
		return NULL;

	$record['password'] = thiscrypt($passwd);

	if ($body)
		$record['body'] = ($append) ? $record['body'].$body : $body;

	return dbputuser($userid,$record);
}


/* userdel - remove user from database */

function userdel($userid) {

	debug("'$userid'");

	if (!userget($userid))
		return NULL;

	return dbdeluser($userid);
}


/* thiscrypt - encrypt a new or check an existing password */

function thiscrypt($string, $password = '') {

	if (!$password)
		$password = SALT1;

	return crypt($string,$password);
}
