<?php

/* MOD/VISITORS.PHP - "Visitor code" functions

NOTE: Experimental and un-finished. See DOC/README.HTML#VISITORS.

Another almost complete, standalone API.

This is a set of functions to support site visitors who post comments. There 
are two "levels of visitation"; one is just when someone posts a comment and 
enters a username and has cookies enabled. Then a cookie is set named "from" 
with that user name and subsequent comment forms remember the username. 

If the configuration setting 'visitorcode' is non-zero an extra form input 
is provided named "code". If a visitor enters a username and a code, that 
username and code gets entered into the 'visitors' database. Subsequent 
comments with that username without a cookie of that name will have to have 
entered a code that matches the stored code. This means a commenter can 
"own" his/her username. (It ain't perfect yet but it's simplicity and 
potential is the reason why it still exists.)

There is a really simple document database table (see file DATABASE):

	+-----------------------------+
	| name                        | VARCHAR(32) UNIQUE
	+-----------------------------+
	| name:Joe\n                  | MEDIUMTEXT
	| code:$5$8kjDjgkshRfgslgb\n  |
	| \n                          |
	| ip:127.0.0.1\n              |
	| time:1375296768\n           |
	+-----------------------------+

Admin has a simple interface for editing and creating Visitors.
*/

if (!defined('SALT2')) {
	define('SALT2','$1$tempsalt$');
	define('SALTTEMP',TRUE);	// for Admin
}

/* visitorget - lookup from visitor store; returns visitor record or 0 */

function visitorget($from) {

	debug("'$from'");

	if ($from === '') return '';
	if (strtolower($from) == 'nobody') return 0;

	$record = dbreadvisitor($from);

	if ($record) {
		// need to validate the record; if a future version adds
		// required data, we need to make sure the old records
		// are updated here... maybe
	}

	return $record;
}


/* visitornew - create a new visitor record */

// $code is already encrypted; $body and $create are for Admin
// called by visitor_store() below and by Admin (both need to check for  
// existing)

function visitornew($from, $code, $body = NULL, $creat = TRUE) {

	debug("'$from','$code'");

	// caller validates from data

	if (!$body) {
		$time = time();
		$body = "ip:{$_SERVER['REMOTE_ADDR']}\ntime:$time\n";
	}

	$record['from'] = $from;	// store from inside header; important!
	$record['code'] = $code;
	$record['body'] = $body;

	if (!$creat)
		return $record;

	return dbnewvisitor($from,$record);
}


/* visitorcreat - create a new database entry */

// called by Admin only - record is created from an Admin form ($_POST data)

function visitorcreat($record) {

	debug($record);

	if (trim($record['from']) === "" || trim($record['code']) === "")
		return '';

	$record['code'] = vcrypt($record['code']);

	return dbnewvisitor($record['from'],$record);
}


/* visitorput - update a visitor record */

// called by Admin when editing a visitor record

function visitorput($record) {

	$from = $record['from'];
	debug("'$from'");

	if (!dbreadvisitor($from))
		return FALSE;

	// has Admin updated the code?

	if (!preg_match('/^'.SALT2.'/',$record['code']))
		$record['code'] = vcrypt($record['code']);

	return dbputvisitor($from,$record);
}


/* visitorchksum - validates a record */

// created this due to finding a corrupt record in the database while testing 
// this is not a real "check sum" as it just checks for all fields to exist
// called by Admin

function visitorchksum($record, $test = TRUE) {

	foreach (array('from','code','body') as $k)
		if (!isset($record[$k]) || $record[$k] === "")
			return $k;
# not complete...
	return '';
}


/* visitordata - create visitor record from POST data */

// called by Admin only - which immediately calls visitorcreat() right after

function visitordata() {

	$def = 'from,code,body';
	$record = postvars($def);

	return $record;
}


/* vcrypt - this is for known visitors */

// note that this one's argument is always the encrypted code

function vcrypt($code) {

	return crypt($code,SALT2);
}


/* these functions are called by module DISPLAY.PHP */


/* visitor_check - checks for visitor cookie */

function visitor_check() {

	return cookie_get('from');
}


/* visitor_login - compare from/code pair login from FORM against store */

// FALSE if from not found
// NULL  if code/vcode mis-match,
// ""    if no from or code

function visitor_login($from, $code) {

	debug("'$from','$code'");

	if (!$from || !$code)
		return '';

	if (!($visitor = visitorget($from)))
		return FALSE;

	$vcode = $visitor['code'];

	if ($vcode != vcrypt($code))
		return NULL;

	return $vcode;
}


/* visitor_validate - compare from/code pair from comment FORM against store */

// 1     if cookie and from match
// FALSE if from/cookie mis-match
// NULL  if code/vcode mis-match
// ''    if no visitor

function visitor_validate($from, $code) {

	debug("'$from','$code'");

	$cfrom = cookie_get('from');
	if ($cfrom !== '') {

		// first check is if form from matches cookie from

		if ($from != $cfrom) {
			debug("from '$from' != '$cfrom'");
			return FALSE;
		}

		// then check if the encoded cookie matches (i.e. can someone
		// forge the encryted from?) note 1
# should be just one cookie
		$cdat = cookie_get('fromdat');
		$cdat = cookie_decode($cdat,$cfrom);
		if ($cdat != $cfrom) {
			debug("data '$cdat' != '$cfrom'");
			return FALSE;
		}
		debug('validated');
		return 1;
	}

	// no cookie so see if code

	// if $code is empty, that's okay
	// if $code and not $visitor, that's okay ($code dealt with later)
	// if $code and $visitor and $code != $vcode that's not okay

	if ($from !== '' && $code !== '' && ($visitor = visitorget($from))) {
		$vcode = $visitor['code'];
		if (vcrypt($code) != $vcode) {
			debug("'$code' != '$vcode'");
			return NULL;
		}
		debug('validated');
		return 1;
	}

	return '';
}


/* visitor_store - updates cookie and/or stores from/code pair */

// if visitorcode off just sets cookie from comment FORM; see file VISITORS

function visitor_store($from, $code) {

	debug("'$from','$code'");

# special case if $from has been defaulted (slight "code logic error")
	if (!($n = config('nobody'))) $n = 'nobody';
	if (strtolower($from) == $n)
		return;

	// if have from set cookie (or re-set expire time)

	if ($from !== '') {
		cookie_set('from',$from);
		$c = cookie_encode($from);
		cookie_set('fromdat',$c);
		debug("from cookie: '$c'");
	}

	if (!config('visitorcode'))
		return;

	// if have from/code pair and not stored create new stored pair

	if ($from !== '' && $code !== '' && !visitorget($from))
		visitornew($from,vcrypt($code));
}

/*
NOTES
1 Cookies before and after setup can cause subtle problems. Changing the 
salt invalidates existing cookies, for example.
*/
