<?php

/* MYSQL.PHP -  ALL MySQL database access is done through here
                ALL SQL query strings are in here

   This module will use the mysqli extension if available, otherwise the mysql 
   extenstion will be used (or if the setting 'mysql' is set to non-zero in 
   CONFIG.INI).

   See file MYSQL for more information about this module.
   See file DATABASE for a description of the database design.
   See DAT/SQL.PHP for the table definitions.

   The following functions are defined:

   These are for posts:

   dblistrecords($id)
   dbreadrecord($id)
   dbgetrecord($id)
   dbnewrecord($data)
   dbputrecord($id, $data)
   dbdelrecord($id)

   These are for comments:

   dblistcomments($entryid)
   dbreadcomment($id)
   dbgetcomment($id)
   dbwritecomment($entryid, $record)
   dbputcomment($id, $data)
   dbdelcomment($id)

   These are for users (there is one, Admin):

   dbreaduser($userid)
   dbnewuser($userid, $record)
   dbputuser($userid, $record)

   These are for Visitors (people who post comments):

   dbreadvisitor($from)
   dbnewvisitor($from, $record)
   dbputvisitor($from, $data)
   dbdelvisitor($from)

   The 'list' functions return arrays
   The 'read' functions return records
   The 'get' functions return strings
   The 'new', 'put' and 'del' functions return TRUE or FALSE
   The 'write' functions returns ID

   The naming convention is really odd here as in there are functions with 
   names like _my_mysql_query(). This is due to the "overloading" of the 
   mysql/mysqli functions (which use _mysql_query()). (A later version will 
   split up mysql/mysqli correctly, to be chosen during setup.)

   Some of these functions are an encapsualization of an encapsualization, but 
   that is not as bad as it sounds (well, that may be a little subjective). But 
  also, there may be yet another redesign to eliminate that.
*/


/* this module's configuration function */

// this does the repository/credentials/session stuff all at once automaticlly
// the Admin setup funnction creates the DB_ defines and creates te database
// this function creates the database tables

function _config_mysql() {
	_mysql();			// this function sets the extension to 
					//  use and stores some data
	if (!defined('DB_NAME'))
		return;

	if (!dbconnect(DB_USER,DB_PASS))
		_mysql_stop('unable to connect with server.');

	if (!dbselect(DB_NAME))
		_mysql_stop("database '".DB_NAME."' does not exist.");

	$db_table = config('arg');
	include 'dat/sql.php';		// this references $db_table
	dbtables($db_table,$tables);
}

/*
   This module has two sections. The 'db' functions that are the API for all 
   of the other modules and the '_mysql' functions that are the API for MySQL.
   Yes, it's an encapsualization of an encapsualization but that is good...
   If the code is to use SQLITE for example, an SQLITE API is created and then 
   there is a search/replace of '_mysql'/'_sqlite', and save this file as 
   SQLITE.PHP.
*/


/* dbconnect - connect to MySQL */

function dbconnect($user, $pass) {

	return _mysql('connect',$user,$pass);
# the previous call is more attractive than:
//	$link _mysql_connect($user,$pass);
//	if ($link) _mysql('link',$link);
//	return $link;
# or is it?
}

/* dbselect - select database, create if needed */

// (created during setup)

function dbselect($name, $create = FALSE) {

	if (_mysql_select_db($name) == FALSE) {
		if (!$create) {
			debug("unable to select '$name'");
			return FALSE;
		}
		debug("unable to select '$name', creating");
		$res = _my_mysql_query("CREATE DATABASE $name");
		if (!$res)
			return $res;
		_mysql_select_db($name);
	}

	return TRUE;
}

/* dbtables - will create tables, if allowed, if they do not exist */

// see DB note 1

function dbtables($table, $tabledef) {

	_mysql('table',$table);			// this saves the value
						//  it never changes once set
	if (config('dbcreated'))		// once all tables created
		return;				//  set dbcreated = 1 

	debug("TABLE '$table'");

	if (!_my_mysql_query("SHOW TABLES LIKE '$table'")) {
		debug("TABLE '$table' not found");

		foreach ($tabledef as $sql)
			if (_my_mysql_query($sql) == FALSE)
				_mysql_stop("unable to create '$table'.");
	}
}

/* dbquery - perform a raw query */

function dbquery($sql) {

	return _my_mysql_query($sql);
}

/* dberror - get last error string */

function dberror($connect = FALSE) {

	$error = ($connect) ? _mysql_connect_error() : _mysql_error();
	$error = trim(trim($error),'.');
	$ext = _mysql('extension');
	return "$ext: $error.";
}

/*
   Start of the various function aliases, someday to be replaced; which 
   means like dblistrecords($id) replaced by dbrecord('list',$id) etc.

   See MYSQL note 1.
*/

/* `section` table */

function dblistrecords($id = NULL) {

	return dbrecord('list',$id);
}

function dbreadrecord($id) {

	return dbrecord('read',$id);
}

function dbgetrecord($id) {

	return dbrecord('get',$id);
}

function dbnewrecord(/*mixed*/ $data) {

	return dbrecord('new',NULL,$data);
}

function dbputrecord($id, /*mixed*/ $data) {

	return dbrecord('put',$id,$data);
}

function dbdelrecord($id) {

	return dbrecord('delete',$id);
}

function dbrecord($cmd, $id = NULL, $data = NULL) {

	debug($cmd);

	if ($data) {
		if (is_array($data))
			$data = recordtostring($data);
		$data = _mysql_real_escape_string($data);
	}

	$db_table = _mysql('table');		// get saved value

	$sql = array(
	'count' => "SELECT COUNT(*) FROM `$db_table`",
	'_count' => "SELECT COUNT(*) FROM `$db_table` WHERE id = $id",
	'list' => "SELECT id FROM `$db_table` ORDER BY id",
	'_list' => "SELECT id FROM `$db_table` WHERE id = $id",
	'id' => "SELECT id FROM `$db_table` WHERE id = $id",
	'read' => "SELECT body FROM `$db_table` WHERE id = $id",
	'get' => "SELECT body FROM `$db_table` WHERE id = $id",
// 'get' called by admin only (same as 'read' but returned record is raw)
	'new' => "INSERT INTO `$db_table` (body) VALUES ('$data')",
	'put' => "UPDATE `$db_table` SET body = '$data' WHERE id = $id",
	'delete' => "DELETE FROM `$db_table` WHERE id = $id",
	);

	// adjust query string if $id being supplied
	if ($cmd == 'list' && $id != NULL)
		$cmd = '_list';
	if ($cmd == 'count' && $id != NULL)
		$cmd = '_count';

	// let fail if $cmd incorrect
	$rec = _my_mysql_query($sql[$cmd]);
	if ($rec === TRUE || !$rec)
		return $rec;
// rec can be array() rather than FALSE if 'list/read' a non-existing record 
// (which under normal circumstances ^should never happen^ but could if making
// changes to the code and something not validated got through) and the loose 
// non-zero test catches that; TRUE means no data was returned

	if ($cmd == 'read')
		return stringtorecord($rec[0]);

	if ($cmd == 'get')
		return $rec[0];

	return $rec;
}


/* `comments` table */

function dbcountcomments($entryid) {

	return dbcomment('count',$entryid);
}

function dblistcomments($entryid) {

	return dbcomment('list',$entryid);
}

function dbreadcomment($id) {

	return dbcomment('read',$id);
}

function dbgetcomment($id) {

	return dbcomment('get',$id);
}

function dbwritecomment($entryid, /*mixed*/ $record) {

	return dbcomment('new',$entryid,$record);
}

function dbputcomment($id, $data) {

	return dbcomment('put',$id);
}

function dbdelcomment($id) {

	return dbcomment('delete',$id);
}

function dbcomment($cmd, $id, $data = '') {

	debug($cmd);

	$table = _mysql('table').'_comments';

	if ($data) {
		if (is_array($data))
			$data = recordtostring($data);
		$data = _mysql_real_escape_string($data);
	}

// see MYSQL note 2.
	$sql = array(
	'count' => "SELECT COUNT(*) FROM `$table` WHERE entryid = $id",
	'list' => "SELECT * FROM `$table` WHERE entryid = $id ORDER BY id",
	'read' => "SELECT body FROM `$table` WHERE id = $id",
	'get' => "SELECT body FROM `$table` WHERE id = $id",
	'new' => "INSERT INTO `$table` (body, entryid) VALUES ('$data', '$id')",
	'delete' => "DELETE FROM `$table` WHERE id = $id",
	'put' => "UPDATE `$table` SET body = '$data' WHERE id = $id",
	);

	$rec = _my_mysql_query($sql[$cmd]);
	if ($rec === FALSE)
		return FALSE;

	if ($cmd == 'count')
		return $rec[0];

	if ($cmd == 'read') {
		$rec = stringtorecord($rec[0]);
		$rec['id'] = $id;
		return $rec;
	}

	if ($cmd == 'get')
		return $rec[0];

	if ($cmd == 'new') {
		$sql = "SELECT LAST_INSERT_ID()";
		$rec = _my_mysql_query($sql);
		return $rec[0];
	}

	return $rec;
}

/* the `users` table */

function dbreaduser($userid) {

	return dbuser('read',$userid);
}

function dbnewuser($userid, $record) {

	return dbuser('new',$userid,$record);
}

function dbputuser($userid, $record) {

	return dbuser('put',$userid,$record);
}

function dbdeluser($userid) {

	return dbuser('delete',$userid);
}

function dbuser($cmd, $userid, $data = NULL) {

	debug($cmd);

	if ($data) {
		$data = recordtostring($data);
		$data = _mysql_real_escape_string($data);
	}

	$sql = array(
	'read' => "SELECT body FROM users WHERE userid = '$userid'",
	'new' => "INSERT INTO users (userid, body) VALUES ('$userid', '$data')",
	'put' => "UPDATE users SET body = '$data' WHERE userid = '$userid'",
	'delete' => "DELETE FROM users WHERE userid = '$userid'",
	);

	$rec = _my_mysql_query($sql[$cmd]);
	if ($rec == FALSE || $rec === TRUE)
		return $rec;

	$rec[0] = rtrim($rec[0]);
	$record = stringtorecord($rec[0]);
	return $record;
}

/* the `visitor` table */

function dbreadvisitor($from) {

	return dbvisitor('read',$from);
}

function dbnewvisitor($from, $record) {

	return dbvisitor('new',$from,$record);
}

function dbputvisitor($from, $record) {

	return dbvisitor('put',$from,$record);
}

function dbdelvisitor($from) {

	return dbvisitor('delete',$from);
}

function dbvisitor($cmd, $from = NULL, $data = NULL) {

	debug("'$cmd','$from','$data'");

	if ($data) {
		$data = recordtostring($data);
		$data = _mysql_real_escape_string($data);
	}
	if ($from)
		$from = _mysql_real_escape_string($from);

	$sql = array(
	'read' => "SELECT body FROM visitors WHERE name = '$from'",
	'all' => "SELECT body FROM visitors",
	'count' => "SELECT COUNT(*) FROM visitors",
	'new' => "INSERT INTO visitors (name, body) VALUES ('$from', '$data')",
	'put' => "UPDATE visitors SET body = '$data' WHERE name = '$from'",
	'delete' => "DELETE FROM visitors WHERE name = '$from'",
	'lookup' => "SELECT body FROM visitors WHERE name REGEXP '^[$from]'",
	'range' => "SELECT COUNT(*) FROM visitors WHERE name REGEXP '^[$from]'",
	);

	$res = _my_mysql_query($sql[$cmd]);
	if ($res === FALSE || $res === TRUE)
		return $res;
	if ($res === array())
		return 0;

	if (is_numeric($res[0]))			// COUNT()
		return $res[0];

	foreach ($res as $r)
		$records[] = stringtorecord($r);

	if ($cmd == 'read')
		return $records[0];			// single record

	return $records;
}

/*
DB NOTES
1. The SETUP code already created the database, but this code checks for 
the table (section name) and will create it if it does not exist. This is 
so that after the code is installed a section can be added to SECTIONS.INI and 
it will be created here. A future version will have a "real" section editor in 
Admin to do that) (a "section" here is same as a "category" in blogspeak)
*/


/* _my_mysql_query - query MySQL and fetch rows */

// see MYSQL note 3.

function _my_mysql_query($sql) {

	debug($sql);

	$res = _mysql_query($sql);
	debug($res);
	if ($res === FALSE)
		debug(_mysql_error());

	if (config('debugdbstat'))
		debug(_mysql_stat());

	if ($res === FALSE || $res === TRUE)
		return $res;

//	debug('rows: '._mysql_num_rows($res));

	$rows = array();
	while ($row = _mysql_fetch_row($res))
		$rows[] = str_replace("\r",'',$row[0]);
	_mysql_free_result($res);

//	debug($rows);

	return $rows;
// see MYSQL note 4.
}


/* _mysql_stop - a fatal error has occured */

# this is wrong as it should return $msg and caller should display...

function _mysql_stop($msg) {

	if (debug() && _mysql('link'))
		$msg = 'mysql: ' . _mysql_error();
	else
		$msg = 'MySQL Server: '.$msg;
	debug($msg);
	displayerror('Fatal Error',"<b>$msg</b>");
}


/* the initialize function - also holds the MySQL link and table name */

function _mysql($cmd = NULL, $arg = NULL, $data = NULL) {
static $dblink = NULL;
static $dbtable = '';
static $dbext = 'mysqli';

// this decides which extension to use

	if (!function_exists('_mysql_connect')) {
		_mysql_functions();
		if (!function_exists('mysqli_connect') || config('mysql'))
			$dbext = 'mysql';
	}

// this is an experimental interface (might change)

	if ($cmd)
	switch ($cmd) {
	case 'connect':
		$link = _mysql_connect($arg,$data);
		if ($link) $dblink = $link;
		//debug("$cmd: ".value($link));		// not always needed
		return $link;
	case 'extension':
		if ($arg) $dbext = $arg;
		return $dbext;
	case 'link':
		if ($arg) $dblink = $arg;
		return $dblink;
	case 'table':
		if ($arg) $dbtable = $arg;
		return $dbtable;
	default:
		debug("invalid: '$cmd'");
		return;
	}
}

/* 
   These "load" either the mysql extenstion or the mysqli extention. There 
   is probably a better way to do this... like having real install code.
*/

function _mysql_functions() {

if (!function_exists('mysqli_connect') || config('mysql')) :

	function _mysql_connect($user = DB_USER, $pass = DB_PASS) {
		return @mysql_connect('localhost',$user,$pass);
	}

	function _mysql_connect_error() {
		return mysql_error();
	}

	function _mysql_select_db($name = DB_NAME) {
		return mysql_select_db($name);
	}

	function _mysql_error() {
		return mysql_error();
	}

	function _mysql_query($sql) {
		return mysql_query($sql);
	}

	function _mysql_num_rows($res) {
		return mysql_num_rows($res);
	}

	function _mysql_fetch_row($res) {
		return mysql_fetch_row($res);
	}

	function _mysql_real_escape_string($str) {
		return mysql_real_escape_string($str);
	}

	function _mysql_stat() {
		return mysql_stat();
	}

	function _mysql_free_result($res) {
		mysql_free_result($res);
	}

	function _mysql_close() {
		return mysql_close();
	}

else:

	function _mysql_connect($user = DB_USER, $pass = DB_PASS) {
		return @mysqli_connect('localhost',$user,$pass);
	}

	function _mysql_connect_error() {
		return mysqli_connect_error();
	}

	function _mysql_select_db($name = DB_NAME) {
		return mysqli_select_db(_mysql('link'),$name);
	}

	function _mysql_error() {
		return mysqli_error(_mysql('link'));
	}

	function _mysql_get_server_info() {
		return mysqli_get_server_info(_mysql('link'));
	}

	function _mysql_query($sql) {
		return mysqli_query(_mysql('link'),$sql);
	}

	function _mysql_num_rows($res) {
		return mysqli_num_rows($res);
	}

	function _mysql_fetch_row($res) {
		return mysqli_fetch_row($res);
	}

	function _mysql_real_escape_string($str) {
		return mysqli_real_escape_string(_mysql('link'),$str);
	}

	function _mysql_stat() {
		return mysqli_stat(_mysql('link'));
	}

	function _mysql_free_result($res) {
		mysqli_free_result($res);
	}

	function _mysql_close() {
		return mysqli_close(_mysql('link'));
	}

endif;

}

/*
MYSQL NOTES
1. Might change all WHERE clauses to be just 'id' (instead of being different 
for each table). This might result in a decrease of overall data size.
2. We deliberately do not put '' around $id here. This is the only WHERE clause 
that is from a GET variable, and although we do validate it, if somehow (while 
making changes for example) the data did not get validated -- it should ALWAYS 
be a number -- not having '' means the query WILL FAIL if it has spaces or 
punctuation characters -- which is a good thing! (Which is why, even though a 
few early releases dot not validate that GET variable, there never was an SQL 
Injection vulnerability as, as said, the query always fails if it has spaces or 
punctuation characters -- that is a MYSQL feature and not the code's.
3. The debug calls here can generate many messages but they are very valuable 
when making changes to the code which might introduce a temporary problem (and 
they are useful when trying to learn how MySQL works) -- some have been 
commented out; if you are modifying the code it may be useful to uncomment 
them.
4. SHOW TABLES query returns a resource with no rows if no tables exist.
SELECT COUNT(*) query returns a resource with one row of 0 if there are 
no rows (empty table). SELECT field can also return a resource with no rows if 
no such field exists.
*/
