<?php

/* RULES.PHP - website "rules" manager/handler

   See file doc/rules.html for what this code is for.

   Basically, "rule" is an array of function names with optional space 
   delimited arguments which will be run for a URL with a rule name.
*/

_rules();					// self initialize


/* rules_run - this function runs the site */

// it is initially called by INDEX.PHP but can be called explicitly later

function rules_run($rule) {

	debug("'$rule'");

	$rules = _rules($rule);

	foreach ($rules as $rulestr) {
		$rulestr = rules_getargs($rulestr);

		// convert any super globals
		if (preg_match('/{\$[_A-Z]*\[/',$rulestr)) {
			$rulestr = "\$rulestr = \"$rulestr\";";
			eval($rulestr);
		}

		// get function name and any argument(s)
		$args = explode(' ',$rulestr);
		$name = array_shift($args);

		// look for "special" arguments
		foreach ($args as &$a) {
			$a = str_replace("''",'',$a);
		}
		unset($a);

		debug("'$name'");
		debug($args);

		// the functions can have conditionals

		if ($name[0] == '?') {
			$cond = (isset($r)) ? $r : 0;
			unset($r);
			if (!$cond)
				continue;
			$name = substr($name,1);
		}
		if ($name[0] == ':') {
			$else = (isset($cond)) ? $cond : 1;
			unset($cond);
			if ($else)
				continue;
			$name = substr($name,1);
		}
		if ($name[0] == '!') {
			$not = (isset($r)) ? $r : 1;
			unset($r);
			if ($not)
				continue;
			$name = substr($name,1);
		}

		// call the function

		if (function_exists($name))
			$r = call_user_func_array($name,$args);
		else
			debug("in rule '$rule', function '$name' not found");
	}
}


/* RULE validation functions */

/* validate - checks for validity of object view rules [post] [page] */

function validate() {

	$op = getvar('op');
	$id = getvar('id');

	if ($op == 'post') {
		if (!is_numeric($id) || !dblistrecords($id))
			return FALSE;
	}
	if ($op == 'page') {
		if (!ctype_alnum($id) || !dblistpages($id))
			return FALSE;
	}

// see note 1
	$start = getvar('start');
	$p = config('postsperpage');
	if ($start < 0 || ($start % $p))
		$_GET['start'] = 0;
	// the display code handles $start > number of posts

	return TRUE;
}

/* validate_file - validate file object [file] [view] */

function validate_file() {

	$id = getvar('id');
	$srcdir = config('srcdir');

	if (strpos($id,'..') !== FALSE || !is_file($srcdir.$id))
		return FALSE;

	return TRUE;
}


/* RULES support functions */

/* rules_getargs - replace '$var' in a string with "$_GET['var']" or "''" */

function rules_getargs($_) {

	$re = '/\$([a-z]+)/';
	if (preg_match_all($re,$_,$res)) {
		foreach ($res[0] as $k => $v) {
			$r = (isset($_GET[$res[1][$k]])) ? $_GET[$res[1][$k]] : "''";
			$_ = str_replace($v,$r,$_);
		}
	}
	return $_;
}


/* _rules - initialize the rules data and/or return it */

function _rules($rule = NULL) {
static $rules = array();

	if ($rules == array()) {
//		if (is_file('usr/rules.php'))
//			include 'usr/rules.php';
//		if ($rules == array())
			$rules = rules_load('rules.ini');
		rules_validate();		// validate all URL data
		return;
	}

	if ($rule === NULL)
		return $rules;

	if (!isset($rules[$rule]))
		return $rules[$rules['default']];

	return $rules[$rule];
}

/* validate - URL data validation */

function rules_validate() {

	rules_reduce();				// "reduce" arguments

	if (empty($_GET) || !isset($_GET['op'])) {	// no args or no op
		$_GET['op'] = _rules('default');	//  do the default
		return;
	}

	if (!_rules($_GET['op'])) {		// bad op? should never
		$_GET['op'] = 'error';		//  happen
		return;
	}

	// now op is set and not default, check for possible arguments

	$op = $_GET['op'];
	$id = (isset($_GET['id'])) ? $_GET['id'] : '';
	if (strpos(_rules('requiresid'),$op) !== FALSE && !$id) // *
		$_GET['op'] = 'error';

// * potential bug here if $op is a substring of an id (array lookup better)
}

/* rules_reduce - "reduce" args */

// see note 2

function rules_reduce() {

	if (empty($_GET)) return;

	if ($i = _rules('ignore'))
		$i = explode(' ',$i);

	$rules = _rules();

	foreach ($_GET as $get => $id) {
		if ($i && in_array($get,$i))	// leave these alone
			continue;
		unset($_GET[$get]);
		if (!isset($rules[$get])) {	// if not a rule assume
			$_GET['arg'] = $get;	//  a section name
			continue;
		}
		$_GET['op'] = $get;		// rule is 'op'
		$_GET['id'] = $id;		// it's value is 'id'

		break;				// see note 3
	}
}

/* rules_load - the rules array is an associative array of arrays */

// not too unlike parse_ini_file()

function rules_load($file) {

	$file = @file($file);
	if (!$file)
		user_error("'$file' was not found",E_USER_ERROR); // exits

	$section = '';
	$data = array();

	while (list (, $l) = each($file)) {
		$l = chop($l);
		if ($l == '' || $l[0] == ';')
			continue;

		if (!$section && preg_match('/(.*)\s*=\s*(.*)/',$l,$res)) {
			$data[trim($res[1])] = $res[2];
			continue;
		}

		if (preg_match('/^\[(.*)\]/',$l,$res)) {
			$section = $res[1];
			continue;
		}

		$data[$section][] = $l;
	}

	return $data;
}

/*
NOTES
1. This is because of Google's stupid "parameter testing"! They take it upon 
themselves to use their own values for &start=, like -100, which is total 
hubris and stupidity.
2. There was a design error when this code was started, obvious now with the 
old URLs side by side with the new below (rather embarrassing actually).
The old was 'arg' as the section name, 'op' as the rule (opcode) and 'id' as 
the op's value. New is section name by itself and no value, and rule itself 
set with it's value:

	New		Old
	?post=2		?arg=&op=post&id=2
	?about		?arg=about
	?about&post=2	?arg=about&op=post&id=2

Internally, so no changes were required of the code, 'arg', 'op' and 'id' are 
still used. Having a function to do this URL "mapping" it turns out can be a 
good thing in that ^if another URL scheme is wanted there just needs to be 
another mapping algorithm^ -- like /about/post_title. (Programming should be 
fun.)
3. The break forces the section name to be required first, e.g. ?about&post=1. 
And without, ?post=1&about would work, but then, processing would go on for all 
variables, and why not just make sure all links displayed are "section first".
*/
