<?php

// Translation data support functions that are called by entries in 
// TRANSLATE.PHP. See file GMLP.


function gmlp_h2($s) {
	$s = ucwords(strtolower($s));
	return '<h2>'.$s.'</h2>';
}

function gmlp_htmlstr($s) {
	return '<code>'.htmlentities(strtolower($s)).'</code>';
}

function gmlp_highlightstr($s) {
	return highlight_string($s,TRUE);
}

// highlighed block without the PHP start/end tags

function gmlp_highlightstr_php($s) {
// have to add them:
	$s = "<?php\n$s\n?>";
	$s = highlight_string($s,TRUE);
// have to remove them:
	remove_php_tags($s);	// passed by reference (see below)
	return $s;
}

function gmlp_list($s) {
	return '<li>'.$s;
}

function gmlp_dlist($s) {
	return preg_replace('/(\S+)(\s+)(.*)/','<dt>$1<dd>$3</dd>',$s);
}

function gmlp_pre($s) {
	return '<pre>'.$s.'</pre>';
}

// "What would Geshi do?"

function gmlp_ini($s) {
	if ($s == '') return $s;

	$s = htmlentities($s);

	if (preg_match('/^\s*;/',$s))
		$s = "<span style=\"color:#666;\">$s</span>";
	else
	if (preg_match('/^\s*\[/',$s))
		$s = "<span style=\"color:#000066;\">$s</span>";
	else
	if (preg_match('/(\s*)([a-z]+)\s*=\s*(.*)/',$s,$res)) {
		$s = "{$res[1]}<span style=\"color:#000099;\">{$res[2]}</span>";
		$s.= " <span style=\"color:#000066;\">=</span>";
		if (isset($res[3]))
			$s.= " <span style=\"color:#660066;\">{$res[3]}</span>";
	}
	return $s;
}

// simple PHP strings

function gmlp_php_string($s) {

	$s = gmlp_highlightstr_php($s);
	$c = ini_get('highlight.html');
	// remove stuff not needed
	$s = str_replace("\n",'',$s);
	$s = str_replace("<code><span style=\"color: $c\">",'<code>',$s);
	$s = str_replace('</span></code>','</code>',$s);
	$s = str_replace('<br />','',$s);
	// to avoid some inline translations
	$s = str_replace('\'','&#39;',$s);
	$s = str_replace('$','&#36;',$s);

	return $s;
}

// a more complicated version of gmlp_highlightstr_php()

function gmlp_div_highlightstr_php($s) {
	$s = "<?php\n$s?>";
	$s = highlight_string($s,TRUE);

	$s = str_replace('&lt;?php<br />','',$s);
	// highlight_string() can end differently
	$c = ini_get('highlight.default');
	if (preg_match("/$c\">\?&gt;<\/span>\n/",$s))
		$replace = "<span style=\"color: $c\">?&gt;</span>\n";
	else
		$replace = '<br />?&gt;';
	$s = str_replace($replace,'',$s);

	// remove not needed stuff and reduce size 
	$c = ini_get('highlight.html');
	$s = str_replace("\r",'',$s);
	$s = str_replace("\n",'',$s);
	$s = str_replace("<code><span style=\"color: $c\">",'',$s);
	$s = str_replace('<br /></span></span></code>','</span>',$s);
	$s = str_replace('color: ','color:',$s);
	$s = str_replace('&nbsp;&nbsp;','&nbsp; ',$s);
	// prepare final output
	$s = str_replace('<br />','<br><span class="n"></span>',$s);
	$s = '<div class="php"><span title="highlighting the code will '.
	'exclude the line numbers">PHP code</span>'."\n".'<span class="n">'.
	'</span>'.$s."</div>\n";
	return $s."\n";
}

function remove_php_tags(&$s) {

	$s = str_replace('&lt;?php<br />','',$s);
	// highlight_string() can end differently
	$c = ini_get('highlight.default');
	if (preg_match("/$c\">\?&gt;<\/span>\n/",$s)) {
		$from = "<br /></span><span style=\"color: $c\">?&gt;</span>";
		$to = '</span>';
	}
	else {
		$from = '<br />?&gt;';
		$to = '';
	}
	$s = str_replace($from,$to,$s);
// NOTE
// fooling with the output like this is dangerous as PHP can change the 
// format between releases...
}
